@@ -1,5 +1,7 @@
 <?php
 
+use MediaWiki\MediaWikiServices;
+
 /**
  * ResourceLoaderFileModule for adding the content language Cite CSS
  *
@@ -13,14 +15,15 @@ class CiteCSSFileModule extends ResourceLoaderFileModule {
 		$localBasePath = null,
 		$remoteBasePath = null
 	) {
-		global $wgContLang;
-
+		$contLang = MediaWikiServices::getInstance()->getContentLanguage();
 		parent::__construct( $options, $localBasePath, $remoteBasePath );
 
 		// Get the content language code, and all the fallbacks. The first that
 		// has a ext.cite.style.<lang code>.css file present will be used.
-		$langCodes = array_merge( [ $wgContLang->getCode() ],
-			$wgContLang->getFallbackLanguages() );
+		$langCodes = array_merge(
+			[ $contLang->getCode() ],
+			$contLang->getFallbackLanguages()
+		);
 		foreach ( $langCodes as $lang ) {
 			$langStyleFile = 'ext.cite.style.' . $lang . '.css';
 			$localPath = $this->getLocalPath( $langStyleFile );
