@@ -18,16 +18,6 @@ class ReaderFactory {
 	 * @throws \Exception
 	 */
 	public function getReader( $filename ) {
-		if ( preg_match( '/i18n\.php$/', $filename ) ) {
-			return new PHPReader();
-		}
-
-		// Ugly hack for core i18n files
-		if ( preg_match( '/Messages(.*)\.php$/', $filename ) ) {
-			$code = \Language::getCodeFromFileName( basename( $filename ), 'Messages' );
-			return new PHPReader( $code );
-		}
-
 		if ( preg_match( '/\.json/', $filename ) ) {
 			$code = basename( $filename, '.json' );
 			return new JSONReader( $code );
