@@ -14,10 +14,7 @@ class FinderTest extends \PHPUnit\Framework\TestCase {
 	public function testGetComponents() {
 		$finder = new Finder(
 			[
-				'TranslateSearch' => '/IP/extensions/Translate/TranslateSearch.i18n.php',
-				'Babel' => '/IP/extensions/Babel/Babel.i18n.php',
-			],
-			[
+				'core' => '/IP/languages/i18n',
 				'Babel' => '/IP/extensions/Babel/i18n',
 				'Door' => [
 					'core' => '/IP/extensions/Door/i18n/core',
@@ -31,21 +28,11 @@ class FinderTest extends \PHPUnit\Framework\TestCase {
 
 		$expected = [
 			'repo' => 'mediawiki',
-			'orig' => "file:///IP/languages/messages/Messages*.php",
-			'path' => 'languages/messages/i18n/*.json',
+			'orig' => "file:///IP/languages/i18n/*.json",
+			'path' => 'languages/i18n/*.json',
 		];
-
-		$this->assertArrayHasKey( 'core', $observed );
-		$this->assertEquals( $expected, $observed['core'], 'Core php file' );
-
-		$expected = [
-			'repo' => 'extension',
-			'name' => 'Translate',
-			'orig' => 'file:///IP/extensions/Translate/TranslateSearch.i18n.php',
-			'path' => 'TranslateSearch.i18n.php'
-		];
-		$this->assertArrayHasKey( 'TranslateSearch', $observed );
-		$this->assertEquals( $expected, $observed['TranslateSearch'], 'PHP only extension' );
+		$this->assertArrayHasKey( 'core-0', $observed );
+		$this->assertEquals( $expected, $observed['core-0'], 'Core JSON file' );
 
 		$expected = [
 			'repo' => 'extension',
@@ -54,7 +41,7 @@ class FinderTest extends \PHPUnit\Framework\TestCase {
 			'path' => 'i18n/*.json'
 		];
 		$this->assertArrayHasKey( 'Babel-0', $observed );
-		$this->assertEquals( $expected, $observed['Babel-0'], 'PHP&JSON extension' );
+		$this->assertEquals( $expected, $observed['Babel-0'], 'JSON extension' );
 
 		$expected = [
 			'repo' => 'extension',
