@@ -15,7 +15,7 @@
  * along with MediaViewer.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-( function ( mw, $, oo ) {
+( function () {
 	var C;
 
 	/**
@@ -75,7 +75,7 @@
 		 */
 		this.imageRawMetadata = null;
 	}
-	oo.inheritClass( Canvas, mw.mmv.ui.Element );
+	OO.inheritClass( Canvas, mw.mmv.ui.Element );
 	C = Canvas.prototype;
 
 	/**
@@ -185,12 +185,12 @@
 	C.setUpImageClick = function () {
 		var canvas = this;
 
-		this.handleEvent( 'mmv-reuse-opened', $.proxy( this.handleDialogEvent, this ) );
-		this.handleEvent( 'mmv-reuse-closed', $.proxy( this.handleDialogEvent, this ) );
-		this.handleEvent( 'mmv-download-opened', $.proxy( this.handleDialogEvent, this ) );
-		this.handleEvent( 'mmv-download-closed', $.proxy( this.handleDialogEvent, this ) );
-		this.handleEvent( 'mmv-options-opened', $.proxy( this.handleDialogEvent, this ) );
-		this.handleEvent( 'mmv-options-closed', $.proxy( this.handleDialogEvent, this ) );
+		this.handleEvent( 'mmv-reuse-opened', this.handleDialogEvent.bind( this ) );
+		this.handleEvent( 'mmv-reuse-closed', this.handleDialogEvent.bind( this ) );
+		this.handleEvent( 'mmv-download-opened', this.handleDialogEvent.bind( this ) );
+		this.handleEvent( 'mmv-download-closed', this.handleDialogEvent.bind( this ) );
+		this.handleEvent( 'mmv-options-opened', this.handleDialogEvent.bind( this ) );
+		this.handleEvent( 'mmv-options-closed', this.handleDialogEvent.bind( this ) );
 
 		this.$image.on( 'click.mmv-canvas', function ( e ) {
 			// ignore clicks if the metadata panel or one of the dialogs is open - assume the intent is to
@@ -315,12 +315,16 @@
 			animationLength = 300;
 
 		// The blurred class has an opacity < 1. This animated the image to become fully opaque
+		// FIXME: Use CSS transition
+		// eslint-disable-next-line no-jquery/no-animate
 		this.$image
 			.addClass( 'blurred' )
 			.animate( { opacity: 1.0 }, animationLength );
 
 		// During the same amount of time (animationLength) we animate a blur value from 3.0 to 0.0
 		// We pass that value to an inline CSS Gaussian blur effect
+		// FIXME: Use CSS transition
+		// eslint-disable-next-line no-jquery/no-animate
 		$( { blur: 3.0 } ).animate( { blur: 0.0 }, {
 			duration: animationLength,
 			step: function ( step ) {
@@ -403,6 +407,7 @@
 	 */
 	C.getDimensions = function ( forFullscreen ) {
 		var $window = $( window ),
+			// eslint-disable-next-line no-jquery/no-global-selector
 			$aboveFold = $( '.mw-mmv-above-fold' ),
 			isFullscreened = !!$aboveFold.closest( '.jq-fullscreened' ).length,
 			// Don't rely on this.$imageWrapper's sizing because it's fragile.
@@ -464,4 +469,4 @@
 	};
 
 	mw.mmv.ui.Canvas = Canvas;
-}( mediaWiki, jQuery, OO ) );
+}() );
