@@ -26,18 +26,19 @@
 
 namespace MediaWiki\Storage;
 
+use AppendIterator;
 use DBAccessObjectUtils;
-use ExternalStore;
 use IDBAccessObject;
 use IExpiringStore;
 use InvalidArgumentException;
-use Language;
 use MWException;
+use StatusValue;
 use WANObjectCache;
+use ExternalStoreAccess;
 use Wikimedia\Assert\Assert;
-use Wikimedia\Rdbms\Database;
+use Wikimedia\AtEase\AtEase;
 use Wikimedia\Rdbms\IDatabase;
-use Wikimedia\Rdbms\LoadBalancer;
+use Wikimedia\Rdbms\ILoadBalancer;
 
 /**
  * Service for storing and loading Content objects.
@@ -53,19 +54,24 @@ class SqlBlobStore implements IDBAccessObject, BlobStore {
 	const TEXT_CACHE_GROUP = 'revisiontext:10';
 
 	/**
-	 * @var LoadBalancer
+	 * @var ILoadBalancer
 	 */
 	private $dbLoadBalancer;
 
+	/**
+	 * @var ExternalStoreAccess
+	 */
+	private $extStoreAccess;
+
 	/**
 	 * @var WANObjectCache
 	 */
 	private $cache;
 
 	/**
-	 * @var bool|string Wiki ID
+	 * @var string|bool DB domain ID of a wiki or false for the local one
 	 */
-	private $wikiId;
+	private $dbDomain;
 
 	/**
 	 * @var int
@@ -82,34 +88,32 @@ class SqlBlobStore implements IDBAccessObject, BlobStore {
 	 */
 	private $legacyEncoding = false;
 
-	/**
-	 * @var Language|null
-	 */
-	private $legacyEncodingConversionLang = null;
-
 	/**
 	 * @var boolean
 	 */
 	private $useExternalStore = false;
 
 	/**
-	 * @param LoadBalancer $dbLoadBalancer A load balancer for acquiring database connections
+	 * @param ILoadBalancer $dbLoadBalancer A load balancer for acquiring database connections
+	 * @param ExternalStoreAccess $extStoreAccess Access layer for external storage
 	 * @param WANObjectCache $cache A cache manager for caching blobs. This can be the local
-	 *        wiki's default instance even if $wikiId refers to a different wiki, since
-	 *        makeGlobalKey() is used to constructed a key that allows cached blobs from the
-	 *        same database to be re-used between wikis. For example, enwiki and frwiki will
-	 *        use the same cache keys for blobs from the wikidatawiki database, regardless of
-	 *        the cache's default key space.
-	 * @param bool|string $wikiId The ID of the target wiki database. Use false for the local wiki.
+	 *        wiki's default instance even if $dbDomain refers to a different wiki, since
+	 *        makeGlobalKey() is used to construct a key that allows cached blobs from the
+	 *        same database to be re-used between wikis. For example, wiki A and wiki B will
+	 *        use the same cache keys for blobs fetched from wiki C, regardless of the
+	 *        wiki-specific default key space.
+	 * @param bool|string $dbDomain The ID of the target wiki database. Use false for the local wiki.
 	 */
 	public function __construct(
-		LoadBalancer $dbLoadBalancer,
+		ILoadBalancer $dbLoadBalancer,
+		ExternalStoreAccess $extStoreAccess,
 		WANObjectCache $cache,
-		$wikiId = false
+		$dbDomain = false
 	) {
 		$this->dbLoadBalancer = $dbLoadBalancer;
+		$this->extStoreAccess = $extStoreAccess;
 		$this->cache = $cache;
-		$this->wikiId = $wikiId;
+		$this->dbDomain = $dbDomain;
 	}
 
 	/**
@@ -151,23 +155,26 @@ class SqlBlobStore implements IDBAccessObject, BlobStore {
 	}
 
 	/**
-	 * @return Language|null The locale to use when decoding from a legacy encoding, or null
-	 *         if handling of legacy encoding is disabled.
+	 * @deprecated since 1.34 No longer needed
+	 * @return null
 	 */
 	public function getLegacyEncodingConversionLang() {
-		return $this->legacyEncodingConversionLang;
+		wfDeprecated( __METHOD__ );
+		return null;
 	}
 
 	/**
+	 * Set the legacy encoding to assume for blobs that do not have the utf-8 flag set.
+	 *
+	 * @note The second parameter, Language $language, was removed in 1.34.
+	 *
 	 * @param string $legacyEncoding The legacy encoding to assume for blobs that are
 	 *        not marked as utf8.
-	 * @param Language $language The locale to use when decoding from a legacy encoding.
 	 */
-	public function setLegacyEncoding( $legacyEncoding, Language $language ) {
+	public function setLegacyEncoding( $legacyEncoding ) {
 		Assert::parameterType( 'string', $legacyEncoding, '$legacyEncoding' );
 
 		$this->legacyEncoding = $legacyEncoding;
-		$this->legacyEncodingConversionLang = $language;
 	}
 
 	/**
@@ -187,7 +194,7 @@ class SqlBlobStore implements IDBAccessObject, BlobStore {
 	}
 
 	/**
-	 * @return LoadBalancer
+	 * @return ILoadBalancer
 	 */
 	private function getDBLoadBalancer() {
 		return $this->dbLoadBalancer;
@@ -200,7 +207,7 @@ class SqlBlobStore implements IDBAccessObject, BlobStore {
 	 */
 	private function getDBConnection( $index ) {
 		$lb = $this->getDBLoadBalancer();
-		return $lb->getConnection( $index, [], $this->wikiId );
+		return $lb->getConnectionRef( $index, [], $this->dbDomain );
 	}
 
 	/**
@@ -220,7 +227,7 @@ class SqlBlobStore implements IDBAccessObject, BlobStore {
 			# Write to external storage if required
 			if ( $this->useExternalStore ) {
 				// Store and get the URL
-				$data = ExternalStore::insertToDefault( $data );
+				$data = $this->extStoreAccess->insert( $data, [ 'domain' => $this->dbDomain ] );
 				if ( !$data ) {
 					throw new BlobAccessException( "Failed to store text to external storage" );
 				}
@@ -270,111 +277,186 @@ class SqlBlobStore implements IDBAccessObject, BlobStore {
 	public function getBlob( $blobAddress, $queryFlags = 0 ) {
 		Assert::parameterType( 'string', $blobAddress, '$blobAddress' );
 
-		// No negative caching; negative hits on text rows may be due to corrupted replica DBs
+		$error = null;
 		$blob = $this->cache->getWithSetCallback(
 			$this->getCacheKey( $blobAddress ),
 			$this->getCacheTTL(),
-			function ( $unused, &$ttl, &$setOpts ) use ( $blobAddress, $queryFlags ) {
-				list( $index ) = DBAccessObjectUtils::getDBOptions( $queryFlags );
-				$setOpts += Database::getCacheSetOptions( $this->getDBConnection( $index ) );
-
-				return $this->fetchBlob( $blobAddress, $queryFlags );
+			function ( $unused, &$ttl, &$setOpts ) use ( $blobAddress, $queryFlags, &$error ) {
+				// Ignore $setOpts; blobs are immutable and negatives are not cached
+				list( $result, $errors ) = $this->fetchBlobs( [ $blobAddress ], $queryFlags );
+				// No negative caching; negative hits on text rows may be due to corrupted replica DBs
+				$error = $errors[$blobAddress] ?? null;
+				return $result[$blobAddress];
 			},
 			[ 'pcGroup' => self::TEXT_CACHE_GROUP, 'pcTTL' => IExpiringStore::TTL_PROC_LONG ]
 		);
 
-		if ( $blob === false ) {
-			throw new BlobAccessException( 'Failed to load blob from address ' . $blobAddress );
+		if ( $error ) {
+			throw new BlobAccessException( $error );
 		}
 
+		Assert::postcondition( is_string( $blob ), 'Blob must not be null' );
 		return $blob;
 	}
 
+	/**
+	 * A batched version of BlobStore::getBlob.
+	 *
+	 * @param string[] $blobAddresses An array of blob addresses.
+	 * @param int $queryFlags See IDBAccessObject.
+	 * @throws BlobAccessException
+	 * @return StatusValue A status with a map of blobAddress => binary blob data or null
+	 *         if fetching the blob has failed. Fetch failures errors are the
+	 *         warnings in the status object.
+	 * @since 1.34
+	 */
+	public function getBlobBatch( $blobAddresses, $queryFlags = 0 ) {
+		$errors = null;
+		$addressByCacheKey = $this->cache->makeMultiKeys(
+			$blobAddresses,
+			function ( $blobAddress ) {
+				return $this->getCacheKey( $blobAddress );
+			}
+		);
+		$blobsByCacheKey = $this->cache->getMultiWithUnionSetCallback(
+			$addressByCacheKey,
+			$this->getCacheTTL(),
+			function ( array $blobAddresses, array &$ttls, array &$setOpts ) use ( $queryFlags, &$errors ) {
+				// Ignore $setOpts; blobs are immutable and negatives are not cached
+				list( $result, $errors ) = $this->fetchBlobs( $blobAddresses, $queryFlags );
+				return $result;
+			},
+			[ 'pcGroup' => self::TEXT_CACHE_GROUP, 'pcTTL' => IExpiringStore::TTL_PROC_LONG ]
+		);
+
+		// Remap back to incoming blob addresses. The return value of the
+		// WANObjectCache::getMultiWithUnionSetCallback is keyed on the internal
+		// keys from WANObjectCache::makeMultiKeys, so we need to remap them
+		// before returning to the client.
+		$blobsByAddress = [];
+		foreach ( $blobsByCacheKey as $cacheKey => $blob ) {
+			$blobsByAddress[ $addressByCacheKey[ $cacheKey ] ] = $blob !== false ? $blob : null;
+		}
+
+		$result = StatusValue::newGood( $blobsByAddress );
+		if ( $errors ) {
+			foreach ( $errors as $error ) {
+				$result->warning( 'internalerror', $error );
+			}
+		}
+		return $result;
+	}
+
 	/**
 	 * MCR migration note: this corresponds to Revision::fetchText
 	 *
-	 * @param string $blobAddress
+	 * @param string[] $blobAddresses
 	 * @param int $queryFlags
 	 *
 	 * @throws BlobAccessException
-	 * @return string|false
+	 * @return array [ $result, $errors ] A map of blob addresses to successfully fetched blobs
+	 *         or false if fetch failed, plus and array of errors
 	 */
-	private function fetchBlob( $blobAddress, $queryFlags ) {
-		list( $schema, $id, ) = self::splitBlobAddress( $blobAddress );
+	private function fetchBlobs( $blobAddresses, $queryFlags ) {
+		$textIdToBlobAddress = [];
+		$result = [];
+		$errors = [];
+		foreach ( $blobAddresses as $blobAddress ) {
+			list( $schema, $id ) = self::splitBlobAddress( $blobAddress );
+			//TODO: MCR: also support 'ex' schema with ExternalStore URLs, plus flags encoded in the URL!
+			if ( $schema === 'tt' ) {
+				$textId = intval( $id );
+				$textIdToBlobAddress[$textId] = $blobAddress;
+			} else {
+				$errors[$blobAddress] = "Unknown blob address schema: $schema";
+				$result[$blobAddress] = false;
+				continue;
+			}
 
-		//TODO: MCR: also support 'ex' schema with ExternalStore URLs, plus flags encoded in the URL!
-		if ( $schema === 'tt' ) {
-			$textId = intval( $id );
-		} else {
-			// XXX: change to better exceptions! That makes migration more difficult, though.
-			throw new BlobAccessException( "Unknown blob address schema: $schema" );
+			if ( !$textId || $id !== (string)$textId ) {
+				$errors[$blobAddress] = "Bad blob address: $blobAddress";
+				$result[$blobAddress] = false;
+			}
 		}
 
-		if ( !$textId || $id !== (string)$textId ) {
-			// XXX: change to better exceptions! That makes migration more difficult, though.
-			throw new BlobAccessException( "Bad blob address: $blobAddress" );
+		$textIds = array_keys( $textIdToBlobAddress );
+		if ( !$textIds ) {
+			return [ $result, $errors ];
 		}
-
 		// Callers doing updates will pass in READ_LATEST as usual. Since the text/blob tables
 		// do not normally get rows changed around, set READ_LATEST_IMMUTABLE in those cases.
 		$queryFlags |= DBAccessObjectUtils::hasFlags( $queryFlags, self::READ_LATEST )
 			? self::READ_LATEST_IMMUTABLE
 			: 0;
-
 		list( $index, $options, $fallbackIndex, $fallbackOptions ) =
 			DBAccessObjectUtils::getDBOptions( $queryFlags );
-
 		// Text data is immutable; check replica DBs first.
-		$row = $this->getDBConnection( $index )->selectRow(
+		$dbConnection = $this->getDBConnection( $index );
+		$rows = $dbConnection->select(
 			'text',
-			[ 'old_text', 'old_flags' ],
-			[ 'old_id' => $textId ],
+			[ 'old_id', 'old_text', 'old_flags' ],
+			[ 'old_id' => $textIds ],
 			__METHOD__,
 			$options
 		);
 
-		// Fallback to DB_MASTER in some cases if the row was not found, using the appropriate
+		// Fallback to DB_MASTER in some cases if not all the rows were found, using the appropriate
 		// options, such as FOR UPDATE to avoid missing rows due to REPEATABLE-READ.
-		if ( !$row && $fallbackIndex !== null ) {
-			$row = $this->getDBConnection( $fallbackIndex )->selectRow(
+		if ( $dbConnection->numRows( $rows ) !== count( $textIds ) && $fallbackIndex !== null ) {
+			$fetchedTextIds = [];
+			foreach ( $rows as $row ) {
+				$fetchedTextIds[] = $row->old_id;
+			}
+			$missingTextIds = array_diff( $textIds, $fetchedTextIds );
+			$dbConnection = $this->getDBConnection( $fallbackIndex );
+			$rowsFromFallback = $dbConnection->select(
 				'text',
-				[ 'old_text', 'old_flags' ],
-				[ 'old_id' => $textId ],
+				[ 'old_id', 'old_text', 'old_flags' ],
+				[ 'old_id' => $missingTextIds ],
 				__METHOD__,
 				$fallbackOptions
 			);
+			$appendIterator = new AppendIterator();
+			$appendIterator->append( $rows );
+			$appendIterator->append( $rowsFromFallback );
+			$rows = $appendIterator;
 		}
 
-		if ( !$row ) {
-			wfWarn( __METHOD__ . ": No text row with ID $textId." );
-			return false;
+		foreach ( $rows as $row ) {
+			$blobAddress = $textIdToBlobAddress[$row->old_id];
+			$blob = $this->expandBlob( $row->old_text, $row->old_flags, $blobAddress );
+			if ( $blob === false ) {
+				$errors[$blobAddress] = "Bad data in text row {$row->old_id}.";
+			}
+			$result[$blobAddress] = $blob;
 		}
 
-		$blob = $this->expandBlob( $row->old_text, $row->old_flags, $blobAddress );
-
-		if ( $blob === false ) {
-			wfLogWarning( __METHOD__ . ": Bad data in text row $textId." );
-			return false;
+		// If we're still missing some of the rows, set errors for missing blobs.
+		if ( count( $result ) !== count( $blobAddresses ) ) {
+			foreach ( $blobAddresses as $blobAddress ) {
+				if ( !isset( $result[$blobAddress ] ) ) {
+					$errors[$blobAddress] = "Unable to fetch blob at $blobAddress";
+					$result[$blobAddress] = false;
+				}
+			}
 		}
-
-		return $blob;
+		return [ $result, $errors ];
 	}
 
 	/**
 	 * Get a cache key for a given Blob address.
 	 *
 	 * The cache key is constructed in a way that allows cached blobs from the same database
-	 * to be re-used between wikis. For example, enwiki and frwiki will use the same cache keys
-	 * for blobs from the wikidatawiki database.
+	 * to be re-used between wikis. For example, wiki A and wiki B will use the same cache keys
+	 * for blobs fetched from wiki C.
 	 *
 	 * @param string $blobAddress
 	 * @return string
 	 */
 	private function getCacheKey( $blobAddress ) {
 		return $this->cache->makeGlobalKey(
-			'BlobStore',
-			'address',
-			$this->dbLoadBalancer->resolveDomainID( $this->wikiId ),
+			'SqlBlobStore-blob',
+			$this->dbLoadBalancer->resolveDomainID( $this->dbDomain ),
 			$blobAddress
 		);
 	}
@@ -417,15 +499,16 @@ class SqlBlobStore implements IDBAccessObject, BlobStore {
 					$this->getCacheKey( $cacheKey ),
 					$this->getCacheTTL(),
 					function () use ( $url, $flags ) {
-						// No negative caching per BlobStore::getBlob()
-						$blob = ExternalStore::fetchFromURL( $url, [ 'wiki' => $this->wikiId ] );
+						// Ignore $setOpts; blobs are immutable and negatives are not cached
+						$blob = $this->extStoreAccess
+							->fetchFromURL( $url, [ 'domain' => $this->dbDomain ] );
 
 						return $blob === false ? false : $this->decompressData( $blob, $flags );
 					},
 					[ 'pcGroup' => self::TEXT_CACHE_GROUP, 'pcTTL' => WANObjectCache::TTL_PROC_LONG ]
 				);
 			} else {
-				$blob = ExternalStore::fetchFromURL( $url, [ 'wiki' => $this->wikiId ] );
+				$blob = $this->extStoreAccess->fetchFromURL( $url, [ 'domain' => $this->dbDomain ] );
 				return $blob === false ? false : $this->decompressData( $blob, $flags );
 			}
 		} else {
@@ -521,14 +604,20 @@ class SqlBlobStore implements IDBAccessObject, BlobStore {
 		}
 
 		// Needed to support old revisions left over from from the 1.4 / 1.5 migration.
-		if ( $blob !== false && $this->legacyEncoding && $this->legacyEncodingConversionLang
+		if ( $blob !== false && $this->legacyEncoding
 			&& !in_array( 'utf-8', $blobFlags ) && !in_array( 'utf8', $blobFlags )
 		) {
 			# Old revisions kept around in a legacy encoding?
 			# Upconvert on demand.
 			# ("utf8" checked for compatibility with some broken
 			#  conversion scripts 2008-12-30)
-			$blob = $this->legacyEncodingConversionLang->iconv( $this->legacyEncoding, 'UTF-8', $blob );
+			# Even with //IGNORE iconv can whine about illegal characters in
+			# *input* string. We just ignore those too.
+			# REF: https://bugs.php.net/bug.php?id=37166
+			# REF: https://phabricator.wikimedia.org/T18885
+			AtEase::suppressWarnings();
+			$blob = iconv( $this->legacyEncoding, 'UTF-8//IGNORE', $blob );
+			AtEase::restoreWarnings();
 		}
 
 		return $blob;
@@ -609,12 +698,14 @@ class SqlBlobStore implements IDBAccessObject, BlobStore {
 	/**
 	 * Splits a blob address into three parts: the schema, the ID, and parameters/flags.
 	 *
+	 * @since 1.33
+	 *
 	 * @param string $address
 	 *
 	 * @throws InvalidArgumentException
 	 * @return array [ $schema, $id, $parameters ], with $parameters being an assoc array.
 	 */
-	private static function splitBlobAddress( $address ) {
+	public static function splitBlobAddress( $address ) {
 		if ( !preg_match( '/^(\w+):(\w+)(\?(.*))?$/', $address, $m ) ) {
 			throw new InvalidArgumentException( "Bad blob address: $address" );
 		}
@@ -627,7 +718,7 @@ class SqlBlobStore implements IDBAccessObject, BlobStore {
 	}
 
 	public function isReadOnly() {
-		if ( $this->useExternalStore && ExternalStore::defaultStoresAreReadOnly() ) {
+		if ( $this->useExternalStore && $this->extStoreAccess->isReadOnly() ) {
 			return true;
 		}
 
