@@ -2,8 +2,6 @@
 /**
  * API for MediaWiki 1.17+
  *
- * Created on May 14, 2010
- *
  * Copyright © 2010 Sam Reed
  * Copyright © 2006 Yuri Astrakhan "<Firstname><Lastname>@gmail.com"
  *
@@ -75,7 +73,7 @@ class ApiQueryIWLinks extends ApiQueryBase {
 			$this->dieContinueUsageIf( count( $cont ) != 3 );
 			$op = $params['dir'] == 'descending' ? '<' : '>';
 			$db = $this->getDB();
-			$iwlfrom = intval( $cont[0] );
+			$iwlfrom = (int)$cont[0];
 			$iwlprefix = $db->addQuotes( $cont[1] );
 			$iwltitle = $db->addQuotes( $cont[2] );
 			$this->addWhere(
