@@ -61,7 +61,7 @@ class ApiQueryQueryPage extends ApiQueryGeneratorBase {
 	 * @param string $name
 	 * @return QueryPage
 	 */
-	private function getSpecialPage( $name ) {
+	private function getSpecialPage( $name ) : QueryPage {
 		$qp = $this->specialPageFactory->getPage( $name );
 		if ( !$qp ) {
 			self::dieDebug(
@@ -122,9 +122,12 @@ class ApiQueryQueryPage extends ApiQueryGeneratorBase {
 
 			$title = Title::makeTitle( $row->namespace, $row->title );
 			if ( is_null( $resultPageSet ) ) {
-				$data = [ 'value' => $row->value ];
-				if ( $qp->usesTimestamps() ) {
-					$data['timestamp'] = wfTimestamp( TS_ISO_8601, $row->value );
+				$data = [];
+				if ( isset( $row->value ) ) {
+					$data['value'] = $row->value;
+					if ( $qp->usesTimestamps() ) {
+						$data['timestamp'] = wfTimestamp( TS_ISO_8601, $row->value );
+					}
 				}
 				self::addTitleInfo( $data, $title );
 
