@@ -39,9 +39,7 @@ class LCStoreStaticArray implements LCStore {
 	private $directory;
 
 	public function __construct( $conf = [] ) {
-		global $wgCacheDirectory;
-
-		$this->directory = $conf['directory'] ?? $wgCacheDirectory;
+		$this->directory = $conf['directory'];
 	}
 
 	public function startWrite( $code ) {
@@ -123,6 +121,8 @@ class LCStoreStaticArray implements LCStore {
 			'Generated by LCStoreStaticArray.php -- do not edit!'
 		);
 		file_put_contents( $this->fname, $out );
+		// Release the data to manage the memory in rebuildLocalisationCache
+		unset( $this->data[$this->currentLang] );
 		$this->currentLang = null;
 		$this->fname = null;
 	}
