@@ -22,6 +22,7 @@ namespace MediaWiki\Edit;
 
 use Content;
 use ParserOptions;
+use RuntimeException;
 use ParserOutput;
 
 /**
@@ -32,7 +33,6 @@ use ParserOutput;
  * @since 1.30
  */
 class PreparedEdit {
-
 	/**
 	 * Time this prepared edit was made
 	 *
@@ -73,7 +73,7 @@ class PreparedEdit {
 	 *
 	 * @var ParserOutput|null
 	 */
-	public $output;
+	private $canonicalOutput;
 
 	/**
 	 * Content that is being saved (before PST)
@@ -89,4 +89,36 @@ class PreparedEdit {
 	 */
 	public $oldContent;
 
+	/**
+	 * Lazy-loading callback to get canonical ParserOutput object
+	 *
+	 * @var callable
+	 */
+	public $parserOutputCallback;
+
+	/**
+	 * @return ParserOutput Canonical parser output
+	 */
+	public function getOutput() {
+		if ( !$this->canonicalOutput ) {
+			$this->canonicalOutput = call_user_func( $this->parserOutputCallback );
+		}
+
+		return $this->canonicalOutput;
+	}
+
+	/**
+	 * Fetch the ParserOutput via a lazy-loaded callback (for backwards compatibility).
+	 *
+	 * @deprecated since 1.33
+	 * @param string $name
+	 * @return mixed
+	 */
+	function __get( $name ) {
+		if ( $name === 'output' ) {
+			return $this->getOutput();
+		}
+
+		throw new RuntimeException( "Undefined field $name." );
+	}
 }
