@@ -32,9 +32,9 @@ class ThrottledError extends ErrorPageError {
 		);
 	}
 
-	public function report() {
+	public function report( $action = ErrorPageError::SEND_OUTPUT ) {
 		global $wgOut;
 		$wgOut->setStatusCode( 429 );
-		parent::report();
+		parent::report( $action );
 	}
 }
