@@ -3,16 +3,19 @@
  * Wrapper for Html::namespaceSelector to use in HTMLForm
  */
 class HTMLSelectNamespace extends HTMLFormField {
+
+	/** @var string|null */
+	protected $mAllValue;
+
 	public function __construct( $params ) {
 		parent::__construct( $params );
 
 		$this->mAllValue = array_key_exists( 'all', $params )
 			? $params['all']
 			: 'all';
-
 	}
 
-	function getInputHTML( $value ) {
+	public function getInputHTML( $value ) {
 		return Html::namespaceSelector(
 			[
 				'selected' => $value,
