@@ -30,7 +30,6 @@ use Psr\Log\LoggerAwareInterface;
 use Psr\Log\LoggerInterface;
 use Psr\Log\NullLogger;
 use Wikimedia\ScopedCallback;
-use Wikimedia\WaitConditionLoop;
 
 /**
  * Class representing a cache/ephemeral data store
@@ -50,75 +49,53 @@ use Wikimedia\WaitConditionLoop;
  * For any given instance, methods like lock(), unlock(), merge(), and set() with WRITE_SYNC
  * should semantically operate over its entire access scope; any nodes/threads in that scope
  * should serialize appropriately when using them. Likewise, a call to get() with READ_LATEST
- * from one node in its access scope should reflect the prior changes of any other node its access
- * scope. Any get() should reflect the changes of any prior set() with WRITE_SYNC.
+ * from one node in its access scope should reflect the prior changes of any other node its
+ * access scope. Any get() should reflect the changes of any prior set() with WRITE_SYNC.
+ *
+ * Subclasses should override the default "segmentationSize" field with an appropriate value.
+ * The value should not be larger than what the storage backend (by default) supports. It also
+ * should be roughly informed by common performance bottlenecks (e.g. values over a certain size
+ * having poor scalability). The same goes for the "segmentedValueMaxSize" member, which limits
+ * the maximum size and chunk count (indirectly) of values.
  *
  * @ingroup Cache
  */
-abstract class BagOStuff implements IExpiringStore, LoggerAwareInterface {
-	/** @var array[] Lock tracking */
-	protected $locks = [];
-	/** @var int ERR_* class constant */
-	protected $lastError = self::ERR_NONE;
-	/** @var string */
-	protected $keyspace = 'local';
+abstract class BagOStuff implements IExpiringStore, IStoreKeyEncoder, LoggerAwareInterface {
 	/** @var LoggerInterface */
 	protected $logger;
+
 	/** @var callable|null */
 	protected $asyncHandler;
-	/** @var int Seconds */
-	protected $syncTimeout;
+	/** @var int[] Map of (ATTR_* class constant => QOS_* class constant) */
+	protected $attrMap = [];
 
 	/** @var bool */
-	private $debugMode = false;
-	/** @var array */
-	private $duplicateKeyLookups = [];
-	/** @var bool */
-	private $reportDupes = false;
-	/** @var bool */
-	private $dupeTrackScheduled = false;
-
-	/** @var callable[] */
-	protected $busyCallbacks = [];
+	protected $debugMode = false;
 
 	/** @var float|null */
 	private $wallClockOverride;
 
-	/** @var int[] Map of (ATTR_* class constant => QOS_* class constant) */
-	protected $attrMap = [];
-
-	/** Bitfield constants for get()/getMulti() */
-	const READ_LATEST = 1; // use latest data for replicated stores
-	const READ_VERIFIED = 2; // promise that caller can tell when keys are stale
-	/** Bitfield constants for set()/merge() */
-	const WRITE_SYNC = 4; // synchronously write to all locations for replicated stores
-	const WRITE_CACHE_ONLY = 8; // Only change state of the in-memory cache
+	/** Bitfield constants for get()/getMulti(); these are only advisory */
+	const READ_LATEST = 1; // if supported, avoid reading stale data due to replication
+	const READ_VERIFIED = 2; // promise that the caller handles detection of staleness
+	/** Bitfield constants for set()/merge(); these are only advisory */
+	const WRITE_SYNC = 4; // if supported, block until the write is fully replicated
+	const WRITE_CACHE_ONLY = 8; // only change state of the in-memory cache
+	const WRITE_ALLOW_SEGMENTS = 16; // allow partitioning of the value if it is large
+	const WRITE_PRUNE_SEGMENTS = 32; // delete all the segments if the value is partitioned
+	const WRITE_BACKGROUND = 64; // if supported, do not block on completion until the next read
 
 	/**
-	 * $params include:
+	 * Parameters include:
 	 *   - logger: Psr\Log\LoggerInterface instance
-	 *   - keyspace: Default keyspace for $this->makeKey()
 	 *   - asyncHandler: Callable to use for scheduling tasks after the web request ends.
 	 *      In CLI mode, it should run the task immediately.
-	 *   - reportDupes: Whether to emit warning log messages for all keys that were
-	 *      requested more than once (requires an asyncHandler).
-	 *   - syncTimeout: How long to wait with WRITE_SYNC in seconds.
 	 * @param array $params
+	 * @phan-param array{logger?:Psr\Log\LoggerInterface,asyncHandler?:callable} $params
 	 */
 	public function __construct( array $params = [] ) {
 		$this->setLogger( $params['logger'] ?? new NullLogger() );
-
-		if ( isset( $params['keyspace'] ) ) {
-			$this->keyspace = $params['keyspace'];
-		}
-
 		$this->asyncHandler = $params['asyncHandler'] ?? null;
-
-		if ( !empty( $params['reportDupes'] ) && is_callable( $this->asyncHandler ) ) {
-			$this->reportDupes = true;
-		}
-
-		$this->syncTimeout = $params['syncTimeout'] ?? 3;
 	}
 
 	/**
@@ -130,16 +107,17 @@ abstract class BagOStuff implements IExpiringStore, LoggerAwareInterface {
 	}
 
 	/**
-	 * @param bool $bool
+	 * @param bool $enabled
 	 */
-	public function setDebug( $bool ) {
-		$this->debugMode = $bool;
+	public function setDebug( $enabled ) {
+		$this->debugMode = $enabled;
 	}
 
 	/**
 	 * Get an item with the given key, regenerating and setting it if not found
 	 *
-	 * Nothing is stored nor deleted if the callback returns false
+	 * The callback can take $ttl as argument by reference and modify it.
+	 * Nothing is stored nor deleted if the callback returns false.
 	 *
 	 * @param string $key
 	 * @param int $ttl Time-to-live (seconds)
@@ -152,11 +130,8 @@ abstract class BagOStuff implements IExpiringStore, LoggerAwareInterface {
 		$value = $this->get( $key, $flags );
 
 		if ( $value === false ) {
-			if ( !is_callable( $callback ) ) {
-				throw new InvalidArgumentException( "Invalid cache miss callback provided." );
-			}
-			$value = call_user_func( $callback );
-			if ( $value !== false ) {
+			$value = $callback( $ttl );
+			if ( $value !== false && $ttl >= 0 ) {
 				$this->set( $key, $value, $ttl, $flags );
 			}
 		}
@@ -177,52 +152,7 @@ abstract class BagOStuff implements IExpiringStore, LoggerAwareInterface {
 	 * @param int $flags Bitfield of BagOStuff::READ_* constants [optional]
 	 * @return mixed Returns false on failure or if the item does not exist
 	 */
-	public function get( $key, $flags = 0 ) {
-		$this->trackDuplicateKeys( $key );
-
-		return $this->doGet( $key, $flags );
-	}
-
-	/**
-	 * Track the number of times that a given key has been used.
-	 * @param string $key
-	 */
-	private function trackDuplicateKeys( $key ) {
-		if ( !$this->reportDupes ) {
-			return;
-		}
-
-		if ( !isset( $this->duplicateKeyLookups[$key] ) ) {
-			// Track that we have seen this key. This N-1 counting style allows
-			// easy filtering with array_filter() later.
-			$this->duplicateKeyLookups[$key] = 0;
-		} else {
-			$this->duplicateKeyLookups[$key] += 1;
-
-			if ( $this->dupeTrackScheduled === false ) {
-				$this->dupeTrackScheduled = true;
-				// Schedule a callback that logs keys processed more than once by get().
-				call_user_func( $this->asyncHandler, function () {
-					$dups = array_filter( $this->duplicateKeyLookups );
-					foreach ( $dups as $key => $count ) {
-						$this->logger->warning(
-							'Duplicate get(): "{key}" fetched {count} times',
-							// Count is N-1 of the actual lookup count
-							[ 'key' => $key, 'count' => $count + 1, ]
-						);
-					}
-				} );
-			}
-		}
-	}
-
-	/**
-	 * @param string $key
-	 * @param int $flags Bitfield of BagOStuff::READ_* constants [optional]
-	 * @param mixed|null &$casToken Token to use for check-and-set comparisons
-	 * @return mixed Returns false on failure or if the item does not exist
-	 */
-	abstract protected function doGet( $key, $flags = 0, &$casToken = null );
+	abstract public function get( $key, $flags = 0 );
 
 	/**
 	 * Set an item
@@ -238,6 +168,10 @@ abstract class BagOStuff implements IExpiringStore, LoggerAwareInterface {
 	/**
 	 * Delete an item
 	 *
+	 * For large values written using WRITE_ALLOW_SEGMENTS, this only deletes the main
+	 * segment list key unless WRITE_PRUNE_SEGMENTS is in the flags. While deleting the segment
+	 * list key has the effect of functionally deleting the key, it leaves unused blobs in cache.
+	 *
 	 * @param string $key
 	 * @return bool True if the item was deleted or not found, false on failure
 	 * @param int $flags Bitfield of BagOStuff::WRITE_* constants
@@ -272,125 +206,32 @@ abstract class BagOStuff implements IExpiringStore, LoggerAwareInterface {
 	 * @return bool Success
 	 * @throws InvalidArgumentException
 	 */
-	public function merge( $key, callable $callback, $exptime = 0, $attempts = 10, $flags = 0 ) {
-		return $this->mergeViaCas( $key, $callback, $exptime, $attempts, $flags );
-	}
-
-	/**
-	 * @see BagOStuff::merge()
-	 *
-	 * @param string $key
-	 * @param callable $callback Callback method to be executed
-	 * @param int $exptime Either an interval in seconds or a unix timestamp for expiry
-	 * @param int $attempts The amount of times to attempt a merge in case of failure
-	 * @param int $flags Bitfield of BagOStuff::WRITE_* constants
-	 * @return bool Success
-	 */
-	protected function mergeViaCas( $key, $callback, $exptime = 0, $attempts = 10, $flags = 0 ) {
-		do {
-			$casToken = null; // passed by reference
-			// Get the old value and CAS token from cache
-			$this->clearLastError();
-			$currentValue = $this->doGet( $key, self::READ_LATEST, $casToken );
-			if ( $this->getLastError() ) {
-				$this->logger->warning(
-					__METHOD__ . ' failed due to I/O error on get() for {key}.',
-					[ 'key' => $key ]
-				);
-
-				return false; // don't spam retries (retry only on races)
-			}
-
-			// Derive the new value from the old value
-			$value = call_user_func( $callback, $this, $key, $currentValue, $exptime );
-			$hadNoCurrentValue = ( $currentValue === false );
-			unset( $currentValue ); // free RAM in case the value is large
-
-			$this->clearLastError();
-			if ( $value === false ) {
-				$success = true; // do nothing
-			} elseif ( $hadNoCurrentValue ) {
-				// Try to create the key, failing if it gets created in the meantime
-				$success = $this->add( $key, $value, $exptime, $flags );
-			} else {
-				// Try to update the key, failing if it gets changed in the meantime
-				$success = $this->cas( $casToken, $key, $value, $exptime, $flags );
-			}
-			if ( $this->getLastError() ) {
-				$this->logger->warning(
-					__METHOD__ . ' failed due to I/O error for {key}.',
-					[ 'key' => $key ]
-				);
-
-				return false; // IO error; don't spam retries
-			}
-		} while ( !$success && --$attempts );
-
-		return $success;
-	}
-
-	/**
-	 * Check and set an item
-	 *
-	 * @param mixed $casToken
-	 * @param string $key
-	 * @param mixed $value
-	 * @param int $exptime Either an interval in seconds or a unix timestamp for expiry
-	 * @param int $flags Bitfield of BagOStuff::WRITE_* constants
-	 * @return bool Success
-	 * @throws Exception
-	 */
-	protected function cas( $casToken, $key, $value, $exptime = 0, $flags = 0 ) {
-		if ( !$this->lock( $key, 0 ) ) {
-			return false; // non-blocking
-		}
-
-		$curCasToken = null; // passed by reference
-		$this->doGet( $key, self::READ_LATEST, $curCasToken );
-		if ( $casToken === $curCasToken ) {
-			$success = $this->set( $key, $value, $exptime, $flags );
-		} else {
-			$this->logger->info(
-				__METHOD__ . ' failed due to race condition for {key}.',
-				[ 'key' => $key ]
-			);
-
-			$success = false; // mismatched or failed
-		}
-
-		$this->unlock( $key );
-
-		return $success;
-	}
+	abstract public function merge(
+		$key,
+		callable $callback,
+		$exptime = 0,
+		$attempts = 10,
+		$flags = 0
+	);
 
 	/**
 	 * Change the expiration on a key if it exists
 	 *
 	 * If an expiry in the past is given then the key will immediately be expired
 	 *
+	 * For large values written using WRITE_ALLOW_SEGMENTS, this only changes the TTL of the
+	 * main segment list key. While lowering the TTL of the segment list key has the effect of
+	 * functionally lowering the TTL of the key, it might leave unused blobs in cache for longer.
+	 * Raising the TTL of such keys is not effective, since the expiration of a single segment
+	 * key effectively expires the entire value.
+	 *
 	 * @param string $key
-	 * @param int $expiry TTL or UNIX timestamp
+	 * @param int $exptime TTL or UNIX timestamp
 	 * @param int $flags Bitfield of BagOStuff::WRITE_* constants (since 1.33)
 	 * @return bool Success Returns false on failure or if the item does not exist
 	 * @since 1.28
 	 */
-	public function changeTTL( $key, $expiry = 0, $flags = 0 ) {
-		$found = false;
-
-		$ok = $this->merge(
-			$key,
-			function ( $cache, $ttl, $currentValue ) use ( &$found ) {
-				$found = ( $currentValue !== false );
-
-				return $currentValue; // nothing is written if this is false
-			},
-			$expiry,
-			1, // 1 attempt
-			$flags
-		);
-
-		return ( $ok && $found );
-	}
+	abstract public function changeTTL( $key, $exptime = 0, $flags = 0 );
 
 	/**
 	 * Acquire an advisory lock on a key string
@@ -403,51 +244,7 @@ abstract class BagOStuff implements IExpiringStore, LoggerAwareInterface {
 	 * @param string $rclass Allow reentry if set and the current lock used this value
 	 * @return bool Success
 	 */
-	public function lock( $key, $timeout = 6, $expiry = 6, $rclass = '' ) {
-		// Avoid deadlocks and allow lock reentry if specified
-		if ( isset( $this->locks[$key] ) ) {
-			if ( $rclass != '' && $this->locks[$key]['class'] === $rclass ) {
-				++$this->locks[$key]['depth'];
-				return true;
-			} else {
-				return false;
-			}
-		}
-
-		$fname = __METHOD__;
-		$expiry = min( $expiry ?: INF, self::TTL_DAY );
-		$loop = new WaitConditionLoop(
-			function () use ( $key, $expiry, $fname ) {
-				$this->clearLastError();
-				if ( $this->add( "{$key}:lock", 1, $expiry ) ) {
-					return WaitConditionLoop::CONDITION_REACHED; // locked!
-				} elseif ( $this->getLastError() ) {
-					$this->logger->warning(
-						$fname . ' failed due to I/O error for {key}.',
-						[ 'key' => $key ]
-					);
-
-					return WaitConditionLoop::CONDITION_ABORTED; // network partition?
-				}
-
-				return WaitConditionLoop::CONDITION_CONTINUE;
-			},
-			$timeout
-		);
-
-		$code = $loop->invoke();
-		$locked = ( $code === $loop::CONDITION_REACHED );
-		if ( $locked ) {
-			$this->locks[$key] = [ 'class' => $rclass, 'depth' => 1 ];
-		} elseif ( $code === $loop::CONDITION_TIMED_OUT ) {
-			$this->logger->warning(
-				"$fname failed due to timeout for {key}.",
-				[ 'key' => $key, 'timeout' => $timeout ]
-			);
-		}
-
-		return $locked;
-	}
+	abstract public function lock( $key, $timeout = 6, $expiry = 6, $rclass = '' );
 
 	/**
 	 * Release an advisory lock on a key string
@@ -455,23 +252,7 @@ abstract class BagOStuff implements IExpiringStore, LoggerAwareInterface {
 	 * @param string $key
 	 * @return bool Success
 	 */
-	public function unlock( $key ) {
-		if ( isset( $this->locks[$key] ) && --$this->locks[$key]['depth'] <= 0 ) {
-			unset( $this->locks[$key] );
-
-			$ok = $this->delete( "{$key}:lock" );
-			if ( !$ok ) {
-				$this->logger->warning(
-					__METHOD__ . ' failed to release lock for {key}.',
-					[ 'key' => $key ]
-				);
-			}
-
-			return $ok;
-		}
-
-		return true;
-	}
+	abstract public function unlock( $key );
 
 	/**
 	 * Get a lightweight exclusive self-unlocking lock
@@ -514,141 +295,117 @@ abstract class BagOStuff implements IExpiringStore, LoggerAwareInterface {
 
 	/**
 	 * Delete all objects expiring before a certain date.
-	 * @param string $date The reference date in MW format
-	 * @param callable|bool $progressCallback Optional, a function which will be called
+	 * @param string|int $timestamp The reference date in MW or TS_UNIX format
+	 * @param callable|null $progress Optional, a function which will be called
 	 *     regularly during long-running operations with the percentage progress
-	 *     as the first parameter.
+	 *     as the first parameter. [optional]
+	 * @param int $limit Maximum number of keys to delete [default: INF]
 	 *
-	 * @return bool Success, false if unimplemented
+	 * @return bool Success; false if unimplemented
 	 */
-	public function deleteObjectsExpiringBefore( $date, $progressCallback = false ) {
-		// stub
-		return false;
-	}
+	abstract public function deleteObjectsExpiringBefore(
+		$timestamp,
+		callable $progress = null,
+		$limit = INF
+	);
 
 	/**
 	 * Get an associative array containing the item for each of the keys that have items.
 	 * @param string[] $keys List of keys
 	 * @param int $flags Bitfield; supports READ_LATEST [optional]
-	 * @return array
+	 * @return mixed[] Map of (key => value) for existing keys
 	 */
-	public function getMulti( array $keys, $flags = 0 ) {
-		$res = [];
-		foreach ( $keys as $key ) {
-			$val = $this->get( $key, $flags );
-			if ( $val !== false ) {
-				$res[$key] = $val;
-			}
-		}
-
-		return $res;
-	}
+	abstract public function getMulti( array $keys, $flags = 0 );
 
 	/**
 	 * Batch insertion/replace
+	 *
+	 * This does not support WRITE_ALLOW_SEGMENTS to avoid excessive read I/O
+	 *
+	 * WRITE_BACKGROUND can be used for bulk insertion where the response is not vital
+	 *
 	 * @param mixed[] $data Map of (key => value)
 	 * @param int $exptime Either an interval in seconds or a unix timestamp for expiry
 	 * @param int $flags Bitfield of BagOStuff::WRITE_* constants (since 1.33)
 	 * @return bool Success
 	 * @since 1.24
 	 */
-	public function setMulti( array $data, $exptime = 0, $flags = 0 ) {
-		$res = true;
-		foreach ( $data as $key => $value ) {
-			if ( !$this->set( $key, $value, $exptime, $flags ) ) {
-				$res = false;
-			}
-		}
-
-		return $res;
-	}
+	abstract public function setMulti( array $data, $exptime = 0, $flags = 0 );
 
 	/**
 	 * Batch deletion
+	 *
+	 * This does not support WRITE_ALLOW_SEGMENTS to avoid excessive read I/O
+	 *
+	 * WRITE_BACKGROUND can be used for bulk deletion where the response is not vital
+	 *
 	 * @param string[] $keys List of keys
 	 * @param int $flags Bitfield of BagOStuff::WRITE_* constants
 	 * @return bool Success
 	 * @since 1.33
 	 */
-	public function deleteMulti( array $keys, $flags = 0 ) {
-		$res = true;
-		foreach ( $keys as $key ) {
-			$res = $this->delete( $key, $flags ) && $res;
-		}
+	abstract public function deleteMulti( array $keys, $flags = 0 );
 
-		return $res;
-	}
+	/**
+	 * Change the expiration of multiple keys that exist
+	 *
+	 * @see BagOStuff::changeTTL()
+	 *
+	 * @param string[] $keys List of keys
+	 * @param int $exptime TTL or UNIX timestamp
+	 * @param int $flags Bitfield of BagOStuff::WRITE_* constants (since 1.33)
+	 * @return bool Success
+	 * @since 1.34
+	 */
+	abstract public function changeTTLMulti( array $keys, $exptime, $flags = 0 );
 
 	/**
 	 * Increase stored value of $key by $value while preserving its TTL
 	 * @param string $key Key to increase
 	 * @param int $value Value to add to $key (default: 1) [optional]
+	 * @param int $flags Bit field of class WRITE_* constants [optional]
 	 * @return int|bool New value or false on failure
 	 */
-	abstract public function incr( $key, $value = 1 );
+	abstract public function incr( $key, $value = 1, $flags = 0 );
 
 	/**
 	 * Decrease stored value of $key by $value while preserving its TTL
 	 * @param string $key
 	 * @param int $value Value to subtract from $key (default: 1) [optional]
+	 * @param int $flags Bit field of class WRITE_* constants [optional]
 	 * @return int|bool New value or false on failure
 	 */
-	public function decr( $key, $value = 1 ) {
-		return $this->incr( $key, - $value );
-	}
+	abstract public function decr( $key, $value = 1, $flags = 0 );
 
 	/**
-	 * Increase stored value of $key by $value while preserving its TTL
+	 * Increase the value of the given key (no TTL change) if it exists or create it otherwise
 	 *
-	 * This will create the key with value $init and TTL $ttl instead if not present
+	 * This will create the key with the value $init and TTL $ttl instead if not present.
+	 * Callers should make sure that both ($init - $value) and $ttl are invariants for all
+	 * operations to any given key. The value of $init should be at least that of $value.
 	 *
-	 * @param string $key
-	 * @param int $ttl
-	 * @param int $value
-	 * @param int $init
+	 * @param string $key Key built via makeKey() or makeGlobalKey()
+	 * @param int $exptime Time-to-live (in seconds) or a UNIX timestamp expiration
+	 * @param int $value Amount to increase the key value by [default: 1]
+	 * @param int|null $init Value to initialize the key to if it does not exist [default: $value]
+	 * @param int $flags Bit field of class WRITE_* constants [optional]
 	 * @return int|bool New value or false on failure
 	 * @since 1.24
 	 */
-	public function incrWithInit( $key, $ttl, $value = 1, $init = 1 ) {
-		$this->clearLastError();
-		$newValue = $this->incr( $key, $value );
-		if ( $newValue === false && !$this->getLastError() ) {
-			// No key set; initialize
-			$newValue = $this->add( $key, (int)$init, $ttl ) ? $init : false;
-			if ( $newValue === false && !$this->getLastError() ) {
-				// Raced out initializing; increment
-				$newValue = $this->incr( $key, $value );
-			}
-		}
-
-		return $newValue;
-	}
+	abstract public function incrWithInit( $key, $exptime, $value = 1, $init = null, $flags = 0 );
 
 	/**
 	 * Get the "last error" registered; clearLastError() should be called manually
 	 * @return int ERR_* constant for the "last error" registry
 	 * @since 1.23
 	 */
-	public function getLastError() {
-		return $this->lastError;
-	}
+	abstract public function getLastError();
 
 	/**
 	 * Clear the "last error" registry
 	 * @since 1.23
 	 */
-	public function clearLastError() {
-		$this->lastError = self::ERR_NONE;
-	}
-
-	/**
-	 * Set the "last error" registry
-	 * @param int $err ERR_* constant
-	 * @since 1.23
-	 */
-	protected function setLastError( $err ) {
-		$this->lastError = $err;
-	}
+	abstract public function clearLastError();
 
 	/**
 	 * Let a callback be run to avoid wasting time on special blocking calls
@@ -670,70 +427,7 @@ abstract class BagOStuff implements IExpiringStore, LoggerAwareInterface {
 	 * @param callable $workCallback
 	 * @since 1.28
 	 */
-	public function addBusyCallback( callable $workCallback ) {
-		$this->busyCallbacks[] = $workCallback;
-	}
-
-	/**
-	 * @param string $text
-	 */
-	protected function debug( $text ) {
-		if ( $this->debugMode ) {
-			$this->logger->debug( "{class} debug: $text", [
-				'class' => static::class,
-			] );
-		}
-	}
-
-	/**
-	 * @param int $exptime
-	 * @return bool
-	 */
-	protected function expiryIsRelative( $exptime ) {
-		return ( $exptime != 0 && $exptime < ( 10 * self::TTL_YEAR ) );
-	}
-
-	/**
-	 * Convert an optionally relative time to an absolute time
-	 * @param int $exptime
-	 * @return int
-	 */
-	protected function convertToExpiry( $exptime ) {
-		if ( $this->expiryIsRelative( $exptime ) ) {
-			return (int)$this->getCurrentTime() + $exptime;
-		} else {
-			return $exptime;
-		}
-	}
-
-	/**
-	 * Convert an optionally absolute expiry time to a relative time. If an
-	 * absolute time is specified which is in the past, use a short expiry time.
-	 *
-	 * @param int $exptime
-	 * @return int
-	 */
-	protected function convertToRelative( $exptime ) {
-		if ( $exptime >= ( 10 * self::TTL_YEAR ) ) {
-			$exptime -= (int)$this->getCurrentTime();
-			if ( $exptime <= 0 ) {
-				$exptime = 1;
-			}
-			return $exptime;
-		} else {
-			return $exptime;
-		}
-	}
-
-	/**
-	 * Check if a value is an integer
-	 *
-	 * @param mixed $value
-	 * @return bool
-	 */
-	protected function isInteger( $value ) {
-		return ( is_int( $value ) || ctype_digit( $value ) );
-	}
+	abstract public function addBusyCallback( callable $workCallback );
 
 	/**
 	 * Construct a cache key.
@@ -743,37 +437,27 @@ abstract class BagOStuff implements IExpiringStore, LoggerAwareInterface {
 	 * @param array $args
 	 * @return string Colon-delimited list of $keyspace followed by escaped components of $args
 	 */
-	public function makeKeyInternal( $keyspace, $args ) {
-		$key = $keyspace;
-		foreach ( $args as $arg ) {
-			$key .= ':' . str_replace( ':', '%3A', $arg );
-		}
-		return strtr( $key, ' ', '_' );
-	}
+	abstract public function makeKeyInternal( $keyspace, $args );
 
 	/**
 	 * Make a global cache key.
 	 *
 	 * @since 1.27
 	 * @param string $class Key class
-	 * @param string|null $component [optional] Key component (starting with a key collection name)
-	 * @return string Colon-delimited list of $keyspace followed by escaped components of $args
+	 * @param string ...$components Key components (starting with a key collection name)
+	 * @return string Colon-delimited list of $keyspace followed by escaped components
 	 */
-	public function makeGlobalKey( $class, $component = null ) {
-		return $this->makeKeyInternal( 'global', func_get_args() );
-	}
+	abstract public function makeGlobalKey( $class, ...$components );
 
 	/**
 	 * Make a cache key, scoped to this instance's keyspace.
 	 *
 	 * @since 1.27
 	 * @param string $class Key class
-	 * @param string|null $component [optional] Key component (starting with a key collection name)
-	 * @return string Colon-delimited list of $keyspace followed by escaped components of $args
+	 * @param string ...$components Key components (starting with a key collection name)
+	 * @return string Colon-delimited list of $keyspace followed by escaped components
 	 */
-	public function makeKey( $class, $component = null ) {
-		return $this->makeKeyInternal( $this->keyspace, func_get_args() );
-	}
+	abstract public function makeKey( $class, ...$components );
 
 	/**
 	 * @param int $flag ATTR_* class constant
@@ -784,13 +468,39 @@ abstract class BagOStuff implements IExpiringStore, LoggerAwareInterface {
 		return $this->attrMap[$flag] ?? self::QOS_UNKNOWN;
 	}
 
+	/**
+	 * @return int|float The chunk size, in bytes, of segmented objects (INF for no limit)
+	 * @since 1.34
+	 */
+	public function getSegmentationSize() {
+		return INF;
+	}
+
+	/**
+	 * @return int|float Maximum total segmented object size in bytes (INF for no limit)
+	 * @since 1.34
+	 */
+	public function getSegmentedValueMaxSize() {
+		return INF;
+	}
+
+	/**
+	 * @param int $field
+	 * @param int $flags
+	 * @return bool
+	 * @since 1.34
+	 */
+	final protected function fieldHasFlags( $field, $flags ) {
+		return ( ( $field & $flags ) === $flags );
+	}
+
 	/**
 	 * Merge the flag maps of one or more BagOStuff objects into a "lowest common denominator" map
 	 *
 	 * @param BagOStuff[] $bags
 	 * @return int[] Resulting flag map (class ATTR_* constant => class QOS_* constant)
 	 */
-	protected function mergeFlagMaps( array $bags ) {
+	final protected function mergeFlagMaps( array $bags ) {
 		$map = [];
 		foreach ( $bags as $bag ) {
 			foreach ( $bag->attrMap as $attr => $rank ) {
@@ -806,15 +516,17 @@ abstract class BagOStuff implements IExpiringStore, LoggerAwareInterface {
 	}
 
 	/**
+	 * @internal For testing only
 	 * @return float UNIX timestamp
 	 * @codeCoverageIgnore
 	 */
-	protected function getCurrentTime() {
+	public function getCurrentTime() {
 		return $this->wallClockOverride ?: microtime( true );
 	}
 
 	/**
-	 * @param float|null &$time Mock UNIX timestamp for testing
+	 * @internal For testing only
+	 * @param float|null &$time Mock UNIX timestamp
 	 * @codeCoverageIgnore
 	 */
 	public function setMockTime( &$time ) {
