@@ -19,10 +19,12 @@
  *
  * @file
  * @author Alexandre Emsenhuber
- * @license http://www.gnu.org/copyleft/gpl.html GNU General Public License 2.0 or later
+ * @license GPL-2.0-or-later
  * @since 1.22
  */
 
+use MediaWiki\MediaWikiServices;
+
 /**
  * This class formats rights log entries.
  *
@@ -30,10 +32,11 @@
  */
 class RightsLogFormatter extends LogFormatter {
 	protected function makePageLink( Title $title = null, $parameters = [], $html = null ) {
-		global $wgContLang, $wgUserrightsInterwikiDelimiter;
+		global $wgUserrightsInterwikiDelimiter;
 
 		if ( !$this->plaintext ) {
-			$text = $wgContLang->ucfirst( $title->getDBkey() );
+			$text = MediaWikiServices::getInstance()->getContentLanguage()->
+				ucfirst( $title->getDBkey() );
 			$parts = explode( $wgUserrightsInterwikiDelimiter, $text, 2 );
 
 			if ( count( $parts ) === 2 ) {
@@ -95,7 +98,7 @@ class RightsLogFormatter extends LogFormatter {
 
 		if ( count( $oldGroups ) ) {
 			$params[3] = [ 'raw' => $this->formatRightsList( $oldGroups,
-				isset( $allParams['oldmetadata'] ) ? $allParams['oldmetadata'] : [] ) ];
+				$allParams['oldmetadata'] ?? [] ) ];
 		} else {
 			$params[3] = $this->msg( 'rightsnone' )->text();
 		}
@@ -103,7 +106,7 @@ class RightsLogFormatter extends LogFormatter {
 			// Array_values is used here because of T44211
 			// see use of array_unique in UserrightsPage::doSaveUserGroups on $newGroups.
 			$params[4] = [ 'raw' => $this->formatRightsList( array_values( $newGroups ),
-				isset( $allParams['newmetadata'] ) ? $allParams['newmetadata'] : [] ) ];
+				$allParams['newmetadata'] ?? [] ) ];
 		} else {
 			$params[4] = $this->msg( 'rightsnone' )->text();
 		}
@@ -176,13 +179,12 @@ class RightsLogFormatter extends LogFormatter {
 			$oldmetadata =& $params['oldmetadata'];
 			// unset old metadata entry to ensure metadata goes at the end of the params array
 			unset( $params['oldmetadata'] );
-			$params['oldmetadata'] = array_map( function( $index ) use ( $params, $oldmetadata ) {
+			$params['oldmetadata'] = array_map( function ( $index ) use ( $params, $oldmetadata ) {
 				$result = [ 'group' => $params['4:array:oldgroups'][$index] ];
 				if ( isset( $oldmetadata[$index] ) ) {
 					$result += $oldmetadata[$index];
 				}
-				$result['expiry'] = ApiResult::formatExpiry( isset( $result['expiry'] ) ?
-					$result['expiry'] : null );
+				$result['expiry'] = ApiResult::formatExpiry( $result['expiry'] ?? null );
 
 				return $result;
 			}, array_keys( $params['4:array:oldgroups'] ) );
@@ -194,13 +196,12 @@ class RightsLogFormatter extends LogFormatter {
 			$newmetadata =& $params['newmetadata'];
 			// unset old metadata entry to ensure metadata goes at the end of the params array
 			unset( $params['newmetadata'] );
-			$params['newmetadata'] = array_map( function( $index ) use ( $params, $newmetadata ) {
+			$params['newmetadata'] = array_map( function ( $index ) use ( $params, $newmetadata ) {
 				$result = [ 'group' => $params['5:array:newgroups'][$index] ];
 				if ( isset( $newmetadata[$index] ) ) {
 					$result += $newmetadata[$index];
 				}
-				$result['expiry'] = ApiResult::formatExpiry( isset( $result['expiry'] ) ?
-					$result['expiry'] : null );
+				$result['expiry'] = ApiResult::formatExpiry( $result['expiry'] ?? null );
 
 				return $result;
 			}, array_keys( $params['5:array:newgroups'] ) );
