@@ -24,15 +24,12 @@
 /**
  * Special handling for category description pages, showing pages,
  * subcategories and file that belong to the category
+ *
+ * @property WikiCategoryPage $mPage Set by overwritten newPage() in this class
  */
 class CategoryPage extends Article {
 	# Subclasses can change this to override the viewer class.
-	protected $mCategoryViewerClass = 'CategoryViewer';
-
-	/**
-	 * @var WikiCategoryPage
-	 */
-	protected $mPage;
+	protected $mCategoryViewerClass = CategoryViewer::class;
 
 	/**
 	 * @param Title $title
