@@ -29,6 +29,8 @@ abstract class PoolCounterWork {
 	protected $type = 'generic';
 	/** @var bool */
 	protected $cacheable = false; // does this override getCachedWork() ?
+	/** @var PoolCounter */
+	private $poolCounter;
 
 	/**
 	 * @param string $type The class of actions to limit concurrency for (task type)
