@@ -26,6 +26,7 @@
  * @ingroup Profiler
  *
  * @since 1.25
+ * @property ProfilerXhprof $collector
  */
 class ProfilerOutputDump extends ProfilerOutput {
 
@@ -45,7 +46,11 @@ class ProfilerOutputDump extends ProfilerOutput {
 
 	public function log( array $stats ) {
 		$data = $this->collector->getRawData();
-		$filename = sprintf( "%s/%s.%s%s", $this->params['outputDir'], uniqid(), $this->collector->getProfileID(), $this->suffix );
+		$filename = sprintf( "%s/%s.%s%s",
+			$this->params['outputDir'],
+			uniqid(),
+			$this->collector->getProfileID(),
+			$this->suffix );
 		file_put_contents( $filename, serialize( $data ) );
 	}
 }
