@@ -58,6 +58,11 @@ class ExtensionDependencyError extends Exception {
 	 */
 	public $missingPhpExtensions = [];
 
+	/**
+	 * @var string[]
+	 */
+	public $missingAbilities = [];
+
 	/**
 	 * @param array $errors Each error has a 'msg' and 'type' key at minimum
 	 */
@@ -75,6 +80,9 @@ class ExtensionDependencyError extends Exception {
 				case 'missing-phpExtension':
 					$this->missingPhpExtensions[] = $info['missing'];
 					break;
+				case 'missing-ability':
+					$this->missingAbilities[] = $info['missing'];
+					break;
 				case 'missing-skins':
 					$this->missingSkins[] = $info['missing'];
 					break;
