@@ -24,13 +24,6 @@
  * used via RevDelRevisionList.
  */
 class RevDelArchivedRevisionItem extends RevDelArchiveItem {
-	public function __construct( $list, $row ) {
-		RevDelItem::__construct( $list, $row );
-
-		$this->revision = Revision::newFromArchiveRow( $row,
-			[ 'page' => $this->list->title->getArticleID() ] );
-	}
-
 	public function getIdField() {
 		return 'ar_rev_id';
 	}
