@@ -19,6 +19,7 @@
  * @ingroup RevisionDelete
  */
 
+use MediaWiki\MediaWikiServices;
 use Wikimedia\Rdbms\IDatabase;
 
 /**
@@ -60,15 +61,17 @@ class RevDelFileList extends RevDelList {
 			$archiveNames[] = $timestamp . '!' . $this->title->getDBkey();
 		}
 
+		$oiQuery = OldLocalFile::getQueryInfo();
 		return $db->select(
-			'oldimage',
-			OldLocalFile::selectFields(),
+			$oiQuery['tables'],
+			$oiQuery['fields'],
 			[
 				'oi_name' => $this->title->getDBkey(),
 				'oi_archive_name' => $archiveNames
 			],
 			__METHOD__,
-			[ 'ORDER BY' => 'oi_timestamp DESC' ]
+			[ 'ORDER BY' => 'oi_timestamp DESC' ],
+			$oiQuery['joins']
 		);
 	}
 
@@ -107,7 +110,8 @@ class RevDelFileList extends RevDelList {
 	}
 
 	public function doPostCommitUpdates( array $visibilityChangeMap ) {
-		$file = wfLocalFile( $this->title );
+		$file = MediaWikiServices::getInstance()->getRepoGroup()->getLocalRepo()
+			->newFile( $this->title );
 		$file->purgeCache();
 		$file->purgeDescription();
 
