@@ -86,8 +86,7 @@ final class SessionManager implements SessionManagerInterface {
 
 	/**
 	 * Get the global SessionManager
-	 * @return SessionManagerInterface
-	 *  (really a SessionManager, but this is to make IDEs less confused)
+	 * @return self
 	 */
 	public static function singleton() {
 		if ( self::$instance === null ) {
@@ -178,6 +177,7 @@ final class SessionManager implements SessionManagerInterface {
 		} else {
 			$store = \ObjectCache::getInstance( $this->config->get( 'SessionCacheType' ) );
 		}
+		$this->logger->debug( 'SessionManager using store ' . get_class( $store ) );
 		$this->store = $store instanceof CachedBagOStuff ? $store : new CachedBagOStuff( $store );
 
 		register_shutdown_function( [ $this, 'shutdown' ] );
@@ -314,11 +314,6 @@ final class SessionManager implements SessionManagerInterface {
 		$user->setToken();
 		$user->saveSettings();
 
-		$authUser = \MediaWiki\Auth\AuthManager::callLegacyAuthPlugin( 'getUserInstance', [ &$user ] );
-		if ( $authUser ) {
-			$authUser->resetAuthToken();
-		}
-
 		foreach ( $this->getProviders() as $provider ) {
 			$provider->invalidateSessionsForUser( $user );
 		}
@@ -326,6 +321,7 @@ final class SessionManager implements SessionManagerInterface {
 
 	public function getVaryHeaders() {
 		// @codeCoverageIgnoreStart
+		// @phan-suppress-next-line PhanUndeclaredConstant
 		if ( defined( 'MW_NO_SESSION' ) && MW_NO_SESSION !== 'warn' ) {
 			return [];
 		}
@@ -334,12 +330,9 @@ final class SessionManager implements SessionManagerInterface {
 			$headers = [];
 			foreach ( $this->getProviders() as $provider ) {
 				foreach ( $provider->getVaryHeaders() as $header => $options ) {
-					if ( !isset( $headers[$header] ) ) {
-						$headers[$header] = [];
-					}
-					if ( is_array( $options ) ) {
-						$headers[$header] = array_unique( array_merge( $headers[$header], $options ) );
-					}
+					# Note that the $options value returned has been deprecated
+					# and is ignored.
+					$headers[$header] = null;
 				}
 			}
 			$this->varyHeaders = $headers;
@@ -349,6 +342,7 @@ final class SessionManager implements SessionManagerInterface {
 
 	public function getVaryCookies() {
 		// @codeCoverageIgnoreStart
+		// @phan-suppress-next-line PhanUndeclaredConstant
 		if ( defined( 'MW_NO_SESSION' ) && MW_NO_SESSION !== 'warn' ) {
 			return [];
 		}
@@ -377,23 +371,6 @@ final class SessionManager implements SessionManagerInterface {
 	 * @{
 	 */
 
-	/**
-	 * Auto-create the given user, if necessary
-	 * @private Don't call this yourself. Let Setup.php do it for you at the right time.
-	 * @deprecated since 1.27, use MediaWiki\Auth\AuthManager::autoCreateUser instead
-	 * @param User $user User to auto-create
-	 * @return bool Success
-	 * @codeCoverageIgnore
-	 */
-	public static function autoCreateUser( User $user ) {
-		wfDeprecated( __METHOD__, '1.27' );
-		return \MediaWiki\Auth\AuthManager::singleton()->autoCreateUser(
-			$user,
-			\MediaWiki\Auth\AuthManager::AUTOCREATE_SOURCE_SESSION,
-			false
-		)->isGood();
-	}
-
 	/**
 	 * Prevent future sessions for the user
 	 *
@@ -435,6 +412,7 @@ final class SessionManager implements SessionManagerInterface {
 				$provider->setConfig( $this->config );
 				$provider->setManager( $this );
 				if ( isset( $this->sessionProviders[(string)$provider] ) ) {
+					// @phan-suppress-next-line PhanTypeSuspiciousStringExpression
 					throw new \UnexpectedValueException( "Duplicate provider name \"$provider\"" );
 				}
 				$this->sessionProviders[(string)$provider] = $provider;
@@ -528,11 +506,10 @@ final class SessionManager implements SessionManagerInterface {
 		}
 
 		if ( count( $retInfos ) > 1 ) {
-			$ex = new \OverflowException(
+			throw new SessionOverflowException(
+				$retInfos,
 				'Multiple sessions for this request tied for top priority: ' . implode( ', ', $retInfos )
 			);
-			$ex->sessionInfos = $retInfos;
-			throw $ex;
 		}
 
 		return $retInfos ? $retInfos[0] : null;
@@ -839,6 +816,7 @@ final class SessionManager implements SessionManagerInterface {
 	public function getSessionFromInfo( SessionInfo $info, WebRequest $request ) {
 		// @codeCoverageIgnoreStart
 		if ( defined( 'MW_NO_SESSION' ) ) {
+			// @phan-suppress-next-line PhanUndeclaredConstant
 			if ( MW_NO_SESSION === 'warn' ) {
 				// Undocumented safety case for converting existing entry points
 				$this->logger->error( 'Sessions are supposed to be disabled for this entry point', [
@@ -964,6 +942,6 @@ final class SessionManager implements SessionManagerInterface {
 		self::$globalSessionRequest = null;
 	}
 
-	/**@}*/
+	/** @} */
 
 }
