@@ -1,4 +1,5 @@
 <?php
+
 use MediaWiki\MediaWikiServices;
 
 /**
@@ -30,7 +31,7 @@ use MediaWiki\MediaWikiServices;
  *
  * @ingroup SpecialPage
  */
-class FileDuplicateSearchPage extends QueryPage {
+class SpecialFileDuplicateSearch extends QueryPage {
 	protected $hash = '', $filename = '';
 
 	/**
@@ -108,7 +109,7 @@ class FileDuplicateSearchPage extends QueryPage {
 		$this->hash = '';
 		$title = Title::newFromText( $this->filename, NS_FILE );
 		if ( $title && $title->getText() != '' ) {
-			$this->file = wfFindFile( $title );
+			$this->file = MediaWikiServices::getInstance()->getRepoGroup()->findFile( $title );
 		}
 
 		$out = $this->getOutput();
