@@ -39,6 +39,11 @@ class SpecialPagesWithProp extends QueryPage {
 	 */
 	private $existingPropNames = null;
 
+	/**
+	 * @var int|null
+	 */
+	private $ns;
+
 	/**
 	 * @var bool
 	 */
@@ -64,6 +69,7 @@ class SpecialPagesWithProp extends QueryPage {
 
 		$request = $this->getRequest();
 		$propname = $request->getVal( 'propname', $par );
+		$this->ns = $request->getIntOrNull( 'namespace' );
 		$this->reverse = $request->getBool( 'reverse' );
 		$this->sortByValue = $request->getBool( 'sortbyvalue' );
 
@@ -78,6 +84,13 @@ class SpecialPagesWithProp extends QueryPage {
 				'label-message' => 'pageswithprop-prop',
 				'required' => true,
 			],
+			'namespace' => [
+				'type' => 'namespaceselect',
+				'name' => 'namespace',
+				'label-message' => 'namespace',
+				'all' => '',
+				'default' => $this->ns,
+			],
 			'reverse' => [
 				'type' => 'check',
 				'name' => 'reverse',
@@ -134,7 +147,7 @@ class SpecialPagesWithProp extends QueryPage {
 	}
 
 	public function getQueryInfo() {
-		return [
+		$query = [
 			'tables' => [ 'page_props', 'page' ],
 			'fields' => [
 				'page_id' => 'pp_page',
@@ -153,6 +166,12 @@ class SpecialPagesWithProp extends QueryPage {
 			],
 			'options' => []
 		];
+
+		if ( $this->ns !== null ) {
+			$query['conds']['page_namespace'] = $this->ns;
+		}
+
+		return $query;
 	}
 
 	function getOrderFields() {
