@@ -43,7 +43,7 @@ class SpecialUpload extends SpecialPage {
 		return true;
 	}
 
-	/** Misc variables **/
+	/** Misc variables */
 
 	/** @var WebRequest|FauxRequest The request this form is supposed to handle */
 	public $mRequest;
@@ -56,21 +56,21 @@ class SpecialUpload extends SpecialPage {
 	public $mLocalFile;
 	public $mUploadClicked;
 
-	/** User input variables from the "description" section **/
+	/** User input variables from the "description" section */
 
 	/** @var string The requested target file name */
 	public $mDesiredDestName;
 	public $mComment;
 	public $mLicense;
 
-	/** User input variables from the root section **/
+	/** User input variables from the root section */
 
 	public $mIgnoreWarning;
 	public $mWatchthis;
 	public $mCopyrightStatus;
 	public $mCopyrightSource;
 
-	/** Hidden variables **/
+	/** Hidden variables */
 
 	public $mDestWarningAck;
 
@@ -84,7 +84,7 @@ class SpecialUpload extends SpecialPage {
 	/** @var bool Subclasses can use this to determine whether a file was uploaded */
 	public $mUploadSuccessful = false;
 
-	/** Text injection points for hooks not using HTMLForm **/
+	/** Text injection points for hooks not using HTMLForm */
 	public $uploadFormTextTop;
 	public $uploadFormTextAfterSummary;
 
@@ -311,16 +311,18 @@ class SpecialUpload extends SpecialPage {
 	protected function showViewDeletedLinks() {
 		$title = Title::makeTitleSafe( NS_FILE, $this->mDesiredDestName );
 		$user = $this->getUser();
+		$permissionManager = MediaWikiServices::getInstance()->getPermissionManager();
 		// Show a subtitle link to deleted revisions (to sysops et al only)
 		if ( $title instanceof Title ) {
 			$count = $title->isDeleted();
-			if ( $count > 0 && $user->isAllowed( 'deletedhistory' ) ) {
+			if ( $count > 0 && $permissionManager->userHasRight( $user, 'deletedhistory' ) ) {
 				$restorelink = $this->getLinkRenderer()->makeKnownLink(
 					SpecialPage::getTitleFor( 'Undelete', $title->getPrefixedText() ),
 					$this->msg( 'restorelink' )->numParams( $count )->text()
 				);
-				$link = $this->msg( $user->isAllowed( 'delete' ) ? 'thisisdeleted' : 'viewdeleted' )
-					->rawParams( $restorelink )->parseAsBlock();
+				$link = $this->msg(
+					$permissionManager->userHasRight( $user, 'delete' ) ? 'thisisdeleted' : 'viewdeleted'
+				)->rawParams( $restorelink )->parseAsBlock();
 				$this->getOutput()->addHTML(
 					Html::rawElement(
 						'div',
@@ -486,7 +488,9 @@ class SpecialUpload extends SpecialPage {
 		// Fetch the file if required
 		$status = $this->mUpload->fetchFile();
 		if ( !$status->isOK() ) {
-			$this->showUploadError( $this->getOutput()->parseAsInterface( $status->getWikiText() ) );
+			$this->showUploadError( $this->getOutput()->parseAsInterface(
+				$status->getWikiText( false, false, $this->getLanguage() )
+			) );
 
 			return;
 		}
@@ -557,7 +561,7 @@ class SpecialUpload extends SpecialPage {
 				$changeTags, $this->getUser() );
 			if ( !$changeTagsStatus->isOK() ) {
 				$this->showUploadError( $this->getOutput()->parseAsInterface(
-					$changeTagsStatus->getWikiText()
+					$changeTagsStatus->getWikiText( false, false, $this->getLanguage() )
 				) );
 
 				return;
@@ -574,7 +578,9 @@ class SpecialUpload extends SpecialPage {
 
 		if ( !$status->isGood() ) {
 			$this->showRecoverableUploadError(
-				$this->getOutput()->parseAsInterface( $status->getWikiText() )
+				$this->getOutput()->parseAsInterface(
+					$status->getWikiText( false, false, $this->getLanguage() )
+				)
 			);
 
 			return;
@@ -669,7 +675,8 @@ class SpecialUpload extends SpecialPage {
 			return true;
 		}
 
-		$local = wfLocalFile( $this->mDesiredDestName );
+		$local = MediaWikiServices::getInstance()->getRepoGroup()->getLocalRepo()
+			->newFile( $this->mDesiredDestName );
 		if ( $local && $local->exists() ) {
 			// We're uploading a new version of an existing file.
 			// No creation, so don't watch it if we're not already.
@@ -689,7 +696,7 @@ class SpecialUpload extends SpecialPage {
 	 */
 	protected function processVerificationError( $details ) {
 		switch ( $details['status'] ) {
-			/** Statuses that only require name changing **/
+			/** Statuses that only require name changing */
 			case UploadBase::MIN_LENGTH_PARTNAME:
 				$this->showRecoverableUploadError( $this->msg( 'minlength1' )->escaped() );
 				break;
@@ -707,7 +714,7 @@ class SpecialUpload extends SpecialPage {
 				$this->showRecoverableUploadError( $this->msg( 'windows-nonascii-filename' )->parse() );
 				break;
 
-			/** Statuses that require reuploading **/
+			/** Statuses that require reuploading */
 			case UploadBase::EMPTY_FILE:
 				$this->showUploadError( $this->msg( 'emptyfile' )->escaped() );
 				break;
@@ -780,7 +787,7 @@ class SpecialUpload extends SpecialPage {
 		}
 	}
 
-	/*** Functions for formatting warnings ***/
+	/** Functions for formatting warnings */
 
 	/**
 	 * Formats a result of UploadBase::getExistsWarning as HTML
