@@ -20,6 +20,7 @@
  */
 
 use MediaWiki\MediaWikiServices;
+use MediaWiki\Linker\LinkRenderer;
 use Wikimedia\Rdbms\FakeResultWrapper;
 
 /**
@@ -45,6 +46,11 @@ class AllMessagesTablePager extends TablePager {
 	 */
 	protected $prefix;
 
+	/**
+	 * @var string
+	 */
+	protected $suffix;
+
 	/**
 	 * @var Language
 	 */
@@ -58,9 +64,12 @@ class AllMessagesTablePager extends TablePager {
 	/**
 	 * @param IContextSource|null $context
 	 * @param FormOptions $opts
+	 * @param LinkRenderer $linkRenderer
 	 */
-	public function __construct( IContextSource $context = null, FormOptions $opts ) {
-		parent::__construct( $context );
+	public function __construct( IContextSource $context = null, FormOptions $opts,
+		LinkRenderer $linkRenderer
+	) {
+		parent::__construct( $context, $linkRenderer );
 
 		$this->mIndexField = 'am_title';
 		// FIXME: Why does this need to be set to DIR_DESCENDING to produce ascending ordering?
@@ -176,11 +185,11 @@ class AllMessagesTablePager extends TablePager {
 	 */
 	function reallyDoQuery( $offset, $limit, $order ) {
 		$asc = ( $order === self::QUERY_ASCENDING );
-		$result = new FakeResultWrapper( [] );
 
 		$messageNames = $this->getAllMessages( $order );
 		$statuses = self::getCustomisedStatuses( $messageNames, $this->langcode, $this->foreign );
 
+		$rows = [];
 		$count = 0;
 		foreach ( $messageNames as $key ) {
 			$customised = isset( $statuses['pages'][$key] );
@@ -188,9 +197,9 @@ class AllMessagesTablePager extends TablePager {
 				( $asc && ( $key < $offset || !$offset ) || !$asc && $key > $offset ) &&
 				( ( $this->prefix && preg_match( $this->prefix, $key ) ) || $this->prefix === false )
 			) {
-				$actual = wfMessage( $key )->inLanguage( $this->lang )->plain();
-				$default = wfMessage( $key )->inLanguage( $this->lang )->useDatabase( false )->plain();
-				$result->result[] = [
+				$actual = $this->msg( $key )->inLanguage( $this->lang )->plain();
+				$default = $this->msg( $key )->inLanguage( $this->lang )->useDatabase( false )->plain();
+				$rows[] = [
 					'am_title' => $key,
 					'am_actual' => $actual,
 					'am_default' => $default,
@@ -205,7 +214,7 @@ class AllMessagesTablePager extends TablePager {
 			}
 		}
 
-		return $result;
+		return new FakeResultWrapper( $rows );
 	}
 
 	protected function getStartBody() {
@@ -235,7 +244,7 @@ class AllMessagesTablePager extends TablePager {
 	}
 
 	function formatValue( $field, $value ) {
-		$linkRenderer = MediaWikiServices::getInstance()->getLinkRenderer();
+		$linkRenderer = $this->getLinkRenderer();
 		switch ( $field ) {
 			case 'am_title' :
 				$title = Title::makeTitle( NS_MEDIAWIKI, $value . $this->suffix );
@@ -256,8 +265,7 @@ class AllMessagesTablePager extends TablePager {
 					$title = $linkRenderer->makeKnownLink( $title, $this->getLanguage()->lcfirst( $value ) );
 				} else {
 					$title = $linkRenderer->makeBrokenLink(
-						$title,
-						$this->getLanguage()->lcfirst( $value )
+						$title, $this->getLanguage()->lcfirst( $value )
 					);
 				}
 				if ( $this->mCurrentRow->am_talk_exists ) {
@@ -355,7 +363,7 @@ class AllMessagesTablePager extends TablePager {
 	}
 
 	function getQueryInfo() {
-		return '';
+		return [];
 	}
 
 }
