@@ -24,6 +24,8 @@
  * @ingroup Pager
  */
 use MediaWiki\MediaWikiServices;
+use MediaWiki\Linker\LinkRenderer;
+use MediaWiki\Revision\RevisionRecord;
 use Wikimedia\Rdbms\IResultWrapper;
 use Wikimedia\Rdbms\FakeResultWrapper;
 use Wikimedia\Rdbms\IDatabase;
@@ -40,12 +42,6 @@ class ContribsPager extends RangeChronologicalPager {
 	 */
 	private $target;
 
-	/**
-	 * @var string Set to "newbie" to list contributions from the most recent 1% registered users.
-	 *  $this->target is ignored then. Defaults to "users".
-	 */
-	private $contribs;
-
 	/**
 	 * @var string|int A single namespace number, or an empty string for all namespaces
 	 */
@@ -102,8 +98,24 @@ class ContribsPager extends RangeChronologicalPager {
 	 */
 	private $templateParser;
 
-	public function __construct( IContextSource $context, array $options ) {
-		parent::__construct( $context );
+	public function __construct( IContextSource $context, array $options,
+		LinkRenderer $linkRenderer = null
+	) {
+		// Set ->target before calling parent::__construct() so
+		// parent can call $this->getIndexField() and get the right result. Set
+		// the rest too just to keep things simple.
+		$this->target = $options['target'] ?? '';
+		$this->namespace = $options['namespace'] ?? '';
+		$this->tagFilter = $options['tagfilter'] ?? false;
+		$this->nsInvert = $options['nsInvert'] ?? false;
+		$this->associated = $options['associated'] ?? false;
+
+		$this->deletedOnly = !empty( $options['deletedOnly'] );
+		$this->topOnly = !empty( $options['topOnly'] );
+		$this->newOnly = !empty( $options['newOnly'] );
+		$this->hideMinor = !empty( $options['hideMinor'] );
+
+		parent::__construct( $context, $linkRenderer );
 
 		$msgs = [
 			'diff',
@@ -116,18 +128,6 @@ class ContribsPager extends RangeChronologicalPager {
 			$this->messages[$msg] = $this->msg( $msg )->escaped();
 		}
 
-		$this->target = $options['target'] ?? '';
-		$this->contribs = $options['contribs'] ?? 'users';
-		$this->namespace = $options['namespace'] ?? '';
-		$this->tagFilter = $options['tagfilter'] ?? false;
-		$this->nsInvert = $options['nsInvert'] ?? false;
-		$this->associated = $options['associated'] ?? false;
-
-		$this->deletedOnly = !empty( $options['deletedOnly'] );
-		$this->topOnly = !empty( $options['topOnly'] );
-		$this->newOnly = !empty( $options['newOnly'] );
-		$this->hideMinor = !empty( $options['hideMinor'] );
-
 		// Date filtering: use timestamp if available
 		$startTimestamp = '';
 		$endTimestamp = '';
@@ -235,6 +235,31 @@ class ContribsPager extends RangeChronologicalPager {
 		return new FakeResultWrapper( $result );
 	}
 
+	/**
+	 * Return the table targeted for ordering and continuation
+	 *
+	 * See T200259 and T221380.
+	 *
+	 * @warning Keep this in sync with self::getQueryInfo()!
+	 *
+	 * @return string
+	 */
+	private function getTargetTable() {
+		$user = User::newFromName( $this->target, false );
+		$ipRangeConds = $user->isAnon() ? $this->getIpRangeConds( $this->mDb, $this->target ) : null;
+		if ( $ipRangeConds ) {
+			return 'ip_changes';
+		} else {
+			$conds = ActorMigration::newMigration()->getWhere( $this->mDb, 'rev_user', $user );
+			if ( isset( $conds['orconds']['actor'] ) ) {
+				// @todo: This will need changing when revision_actor_temp goes away
+				return 'revision_actor_temp';
+			}
+		}
+
+		return 'revision';
+	}
+
 	function getQueryInfo() {
 		$revQuery = Revision::getQueryInfo( [ 'page', 'user' ] );
 		$queryInfo = [
@@ -244,73 +269,28 @@ class ContribsPager extends RangeChronologicalPager {
 			'options' => [],
 			'join_conds' => $revQuery['joins'],
 		];
+		$permissionManager = MediaWikiServices::getInstance()->getPermissionManager();
 
-		if ( $this->contribs == 'newbie' ) {
-			$max = $this->mDb->selectField( 'user', 'max(user_id)', '', __METHOD__ );
-			$queryInfo['conds'][] = $revQuery['fields']['rev_user'] . ' >' . (int)( $max - $max / 100 );
-			# ignore local groups with the bot right
-			# @todo FIXME: Global groups may have 'bot' rights
-			$groupsWithBotPermission = User::getGroupsWithPermission( 'bot' );
-			if ( count( $groupsWithBotPermission ) ) {
-				$queryInfo['tables'][] = 'user_groups';
-				$queryInfo['conds'][] = 'ug_group IS NULL';
-				$queryInfo['join_conds']['user_groups'] = [
-					'LEFT JOIN', [
-						'ug_user = ' . $revQuery['fields']['rev_user'],
-						'ug_group' => $groupsWithBotPermission,
-						'ug_expiry IS NULL OR ug_expiry >= ' .
-							$this->mDb->addQuotes( $this->mDb->timestamp() )
-					]
-				];
-			}
-			// (T140537) Disallow looking too far in the past for 'newbies' queries. If the user requested
-			// a timestamp offset far in the past such that there are no edits by users with user_ids in
-			// the range, we would end up scanning all revisions from that offset until start of time.
-			$queryInfo['conds'][] = 'rev_timestamp > ' .
-				$this->mDb->addQuotes( $this->mDb->timestamp( wfTimestamp() - 30 * 24 * 60 * 60 ) );
+		// WARNING: Keep this in sync with getTargetTable()!
+		$user = User::newFromName( $this->target, false );
+		$ipRangeConds = $user->isAnon() ? $this->getIpRangeConds( $this->mDb, $this->target ) : null;
+		if ( $ipRangeConds ) {
+			$queryInfo['tables'][] = 'ip_changes';
+			$queryInfo['join_conds']['ip_changes'] = [
+				'LEFT JOIN', [ 'ipc_rev_id = rev_id' ]
+			];
+			$queryInfo['conds'][] = $ipRangeConds;
 		} else {
-			$user = User::newFromName( $this->target, false );
-			$ipRangeConds = $user->isAnon() ? $this->getIpRangeConds( $this->mDb, $this->target ) : null;
-			if ( $ipRangeConds ) {
-				$queryInfo['tables'][] = 'ip_changes';
-				/**
-				 * These aliases make `ORDER BY rev_timestamp, rev_id` from {@see getIndexField} and
-				 * {@see getExtraSortFields} use the replicated `ipc_rev_timestamp` and `ipc_rev_id`
-				 * columns from the `ip_changes` table, for more efficient queries.
-				 * @see https://phabricator.wikimedia.org/T200259#4832318
-				 */
-				$queryInfo['fields'] = array_merge(
-					[
-						'rev_timestamp' => 'ipc_rev_timestamp',
-						'rev_id' => 'ipc_rev_id',
-					],
-					array_diff( $queryInfo['fields'], [
-						'rev_timestamp',
-						'rev_id',
-					] )
-				);
-				$queryInfo['join_conds']['ip_changes'] = [
-					'LEFT JOIN', [ 'ipc_rev_id = rev_id' ]
-				];
-				$queryInfo['conds'][] = $ipRangeConds;
+			// tables and joins are already handled by Revision::getQueryInfo()
+			$conds = ActorMigration::newMigration()->getWhere( $this->mDb, 'rev_user', $user );
+			$queryInfo['conds'][] = $conds['conds'];
+			// Force the appropriate index to avoid bad query plans (T189026)
+			if ( isset( $conds['orconds']['actor'] ) ) {
+				// @todo: This will need changing when revision_actor_temp goes away
+				$queryInfo['options']['USE INDEX']['temp_rev_user'] = 'actor_timestamp';
 			} else {
-				// tables and joins are already handled by Revision::getQueryInfo()
-				$conds = ActorMigration::newMigration()->getWhere( $this->mDb, 'rev_user', $user );
-				$queryInfo['conds'][] = $conds['conds'];
-				// Force the appropriate index to avoid bad query plans (T189026)
-				if ( isset( $conds['orconds']['actor'] ) ) {
-					// @todo: This will need changing when revision_comment_temp goes away
-					$queryInfo['options']['USE INDEX']['temp_rev_user'] = 'actor_timestamp';
-					// Alias 'rev_timestamp' => 'revactor_timestamp' so "ORDER BY rev_timestamp" is interpreted to
-					// use revactor_timestamp instead.
-					$queryInfo['fields'] = array_merge(
-						array_diff( $queryInfo['fields'], [ 'rev_timestamp' ] ),
-						[ 'rev_timestamp' => 'revactor_timestamp' ]
-					);
-				} else {
-					$queryInfo['options']['USE INDEX']['revision'] =
-						isset( $conds['orconds']['userid'] ) ? 'user_timestamp' : 'usertext_timestamp';
-				}
+				$queryInfo['options']['USE INDEX']['revision'] =
+					isset( $conds['orconds']['userid'] ) ? 'user_timestamp' : 'usertext_timestamp';
 			}
 		}
 
@@ -334,17 +314,20 @@ class ContribsPager extends RangeChronologicalPager {
 		$queryInfo['conds'] = array_merge( $queryInfo['conds'], $this->getNamespaceCond() );
 
 		// Paranoia: avoid brute force searches (T19342)
-		if ( !$user->isAllowed( 'deletedhistory' ) ) {
-			$queryInfo['conds'][] = $this->mDb->bitAnd( 'rev_deleted', Revision::DELETED_USER ) . ' = 0';
-		} elseif ( !$user->isAllowedAny( 'suppressrevision', 'viewsuppressed' ) ) {
-			$queryInfo['conds'][] = $this->mDb->bitAnd( 'rev_deleted', Revision::SUPPRESSED_USER ) .
-				' != ' . Revision::SUPPRESSED_USER;
+		if ( !$permissionManager->userHasRight( $user, 'deletedhistory' ) ) {
+			$queryInfo['conds'][] = $this->mDb->bitAnd(
+				'rev_deleted', RevisionRecord::DELETED_USER
+				) . ' = 0';
+		} elseif ( !$permissionManager->userHasAnyRight( $user, 'suppressrevision', 'viewsuppressed' ) ) {
+			$queryInfo['conds'][] = $this->mDb->bitAnd(
+				'rev_deleted', RevisionRecord::SUPPRESSED_USER
+				) . ' != ' . RevisionRecord::SUPPRESSED_USER;
 		}
 
-		// For IPv6, we use ipc_rev_timestamp on ip_changes as the index field,
-		// which will be referenced when parsing the results of a query.
-		if ( self::isQueryableRange( $this->target ) ) {
-			$queryInfo['fields'][] = 'ipc_rev_timestamp';
+		// $this->getIndexField() must be in the result rows, as reallyDoQuery() tries to access it.
+		$indexField = $this->getIndexField();
+		if ( $indexField !== 'rev_timestamp' ) {
+			$queryInfo['fields'][] = $indexField;
 		}
 
 		ChangeTags::modifyDisplayQuery(
@@ -374,7 +357,7 @@ class ContribsPager extends RangeChronologicalPager {
 			}
 
 			$associatedNS = $this->mDb->addQuotes(
-				MWNamespace::getAssociated( $this->namespace )
+				MediaWikiServices::getInstance()->getNamespaceInfo()->getAssociated( $this->namespace )
 			);
 
 			return [
@@ -430,8 +413,24 @@ class ContribsPager extends RangeChronologicalPager {
 	 * @return string
 	 */
 	public function getIndexField() {
-		// Note this is run via parent::__construct() *before* $this->target is set!
-		return 'rev_timestamp';
+		// The returned column is used for sorting and continuation, so we need to
+		// make sure to use the right denormalized column depending on which table is
+		// being targeted by the query to avoid bad query plans.
+		// See T200259, T204669, T220991, and T221380.
+		$target = $this->getTargetTable();
+		switch ( $target ) {
+			case 'revision':
+				return 'rev_timestamp';
+			case 'ip_changes':
+				return 'ipc_rev_timestamp';
+			case 'revision_actor_temp':
+				return 'revactor_timestamp';
+			default:
+				wfWarn(
+					__METHOD__ . ": Unknown value '$target' from " . static::class . '::getTargetTable()', 0
+				);
+				return 'rev_timestamp';
+		}
 	}
 
 	/**
@@ -441,13 +440,6 @@ class ContribsPager extends RangeChronologicalPager {
 		return $this->tagFilter;
 	}
 
-	/**
-	 * @return string
-	 */
-	public function getContribs() {
-		return $this->contribs;
-	}
-
 	/**
 	 * @return string
 	 */
@@ -473,8 +465,24 @@ class ContribsPager extends RangeChronologicalPager {
 	 * @return string[]
 	 */
 	protected function getExtraSortFields() {
-		// Note this is run via parent::__construct() *before* $this->target is set!
-		return [ 'rev_id' ];
+		// The returned columns are used for sorting, so we need to make sure
+		// to use the right denormalized column depending on which table is
+		// being targeted by the query to avoid bad query plans.
+		// See T200259, T204669, T220991, and T221380.
+		$target = $this->getTargetTable();
+		switch ( $target ) {
+			case 'revision':
+				return [ 'rev_id' ];
+			case 'ip_changes':
+				return [ 'ipc_rev_id' ];
+			case 'revision_actor_temp':
+				return [ 'revactor_rev' ];
+			default:
+				wfWarn(
+					__METHOD__ . ": Unknown value '$target' from " . static::class . '::getTargetTable()', 0
+				);
+				return [ 'rev_id' ];
+		}
 	}
 
 	protected function doBatchLookups() {
@@ -491,10 +499,7 @@ class ContribsPager extends RangeChronologicalPager {
 			}
 			if ( isset( $row->rev_id ) ) {
 				$this->mParentLens[$row->rev_id] = $row->rev_len;
-				if ( $this->contribs === 'newbie' ) { // multiple users
-					$batch->add( NS_USER, $row->user_name );
-					$batch->add( NS_USER_TALK, $row->user_name );
-				} elseif ( $isIpRange ) {
+				if ( $isIpRange ) {
 					// If this is an IP range, batch the IP's talk page
 					$batch->add( NS_USER_TALK, $row->rev_user_text );
 				}
@@ -568,7 +573,8 @@ class ContribsPager extends RangeChronologicalPager {
 		$classes = [];
 		$attribs = [];
 
-		$linkRenderer = MediaWikiServices::getInstance()->getLinkRenderer();
+		$linkRenderer = $this->getLinkRenderer();
+		$permissionManager = MediaWikiServices::getInstance()->getPermissionManager();
 
 		$page = null;
 		// Create a title for the revision if possible
@@ -594,15 +600,17 @@ class ContribsPager extends RangeChronologicalPager {
 				$topmarktext .= '<span class="mw-uctop">' . $this->messages['uctop'] . '</span>';
 				$classes[] = 'mw-contributions-current';
 				# Add rollback link
-				if ( !$row->page_is_new && $page->quickUserCan( 'rollback', $user )
-					&& $page->quickUserCan( 'edit', $user )
+				if ( !$row->page_is_new &&
+					$permissionManager->quickUserCan( 'rollback', $user, $page ) &&
+					$permissionManager->quickUserCan( 'edit', $user, $page )
 				) {
 					$this->preventClickjacking();
-					$topmarktext .= ' ' . Linker::generateRollback( $rev, $this->getContext() );
+					$topmarktext .= ' ' . Linker::generateRollback( $rev, $this->getContext(),
+						[ 'noBrackets' ] );
 				}
 			}
 			# Is there a visible previous revision?
-			if ( $rev->userCan( Revision::DELETED_TEXT, $user ) && $rev->getParentId() !== 0 ) {
+			if ( $rev->userCan( RevisionRecord::DELETED_TEXT, $user ) && $rev->getParentId() !== 0 ) {
 				$difftext = $linkRenderer->makeKnownLink(
 					$page,
 					new HtmlArmor( $this->messages['diff'] ),
@@ -648,12 +656,9 @@ class ContribsPager extends RangeChronologicalPager {
 			$comment = $lang->getDirMark() . Linker::revComment( $rev, false, true, false );
 			$d = ChangesList::revDateLink( $rev, $user, $lang, $page );
 
-			# Show user names for /newbies as there may be different users.
-			# Note that only unprivileged users have rows with hidden user names excluded.
 			# When querying for an IP range, we want to always show user and user talk links.
 			$userlink = '';
-			if ( ( $this->contribs == 'newbie' && !$rev->isDeleted( Revision::DELETED_USER ) )
-				|| $this->isQueryableRange( $this->target ) ) {
+			if ( $this->isQueryableRange( $this->target ) ) {
 				$userlink = ' <span class="mw-changeslist-separator"></span> '
 					. $lang->getDirMark()
 					. Linker::userLink( $rev->getUser(), $rev->getUserText() );
@@ -712,7 +717,7 @@ class ContribsPager extends RangeChronologicalPager {
 			];
 
 			# Denote if username is redacted for this edit
-			if ( $rev->isDeleted( Revision::DELETED_USER ) ) {
+			if ( $rev->isDeleted( RevisionRecord::DELETED_USER ) ) {
 				$templateParams['rev-deleted-user-contribs'] =
 					$this->msg( 'rev-deleted-user-contribs' )->escaped();
 			}
