@@ -1,9 +1,12 @@
 <?php
 
-use Wikimedia\Rdbms\IDatabase;
 use MediaWiki\Linker\LinkTarget;
+use MediaWiki\Permissions\PermissionManager;
+use MediaWiki\Revision\RevisionRecord;
+use MediaWiki\User\UserIdentity;
 use Wikimedia\Assert\Assert;
-use Wikimedia\Rdbms\LoadBalancer;
+use Wikimedia\Rdbms\IDatabase;
+use Wikimedia\Rdbms\ILoadBalancer;
 
 /**
  * Class performing complex database queries related to WatchedItems.
@@ -52,7 +55,7 @@ class WatchedItemQueryService {
 	const SORT_DESC = 'DESC';
 
 	/**
-	 * @var LoadBalancer
+	 * @var ILoadBalancer
 	 */
 	private $loadBalancer;
 
@@ -68,16 +71,21 @@ class WatchedItemQueryService {
 	/** @var WatchedItemStoreInterface */
 	private $watchedItemStore;
 
+	/** @var PermissionManager */
+	private $permissionManager;
+
 	public function __construct(
-		LoadBalancer $loadBalancer,
+		ILoadBalancer $loadBalancer,
 		CommentStore $commentStore,
 		ActorMigration $actorMigration,
-		WatchedItemStoreInterface $watchedItemStore
+		WatchedItemStoreInterface $watchedItemStore,
+		PermissionManager $permissionManager
 	) {
 		$this->loadBalancer = $loadBalancer;
 		$this->commentStore = $commentStore;
 		$this->actorMigration = $actorMigration;
 		$this->watchedItemStore = $watchedItemStore;
+		$this->permissionManager = $permissionManager;
 	}
 
 	/**
@@ -121,8 +129,8 @@ class WatchedItemQueryService {
 	 *        'end'                 => string (format accepted by wfTimestamp) requires 'dir' option,
 	 *                                 timestamp to end enumerating
 	 *        'watchlistOwner'      => User user whose watchlist items should be listed if different
-	 *                                 than the one specified with $user param,
-	 *                                 requires 'watchlistOwnerToken' option
+	 *                                 than the one specified with $user param, requires
+	 *                                 'watchlistOwnerToken' option
 	 *        'watchlistOwnerToken' => string a watchlist token used to access another user's
 	 *                                 watchlist, used with 'watchlistOwnerToken' option
 	 *        'limit'               => int maximum numbers of items to return
@@ -131,7 +139,7 @@ class WatchedItemQueryService {
 	 *                                 id fields ('rc_cur_id', 'rc_this_oldid', 'rc_last_oldid')
 	 *                                 if false (default)
 	 * @param array|null &$startFrom Continuation value: [ string $rcTimestamp, int $rcId ]
-	 * @return array of pairs ( WatchedItem $watchedItem, string[] $recentChangeInfo ),
+	 * @return array[] Array of pairs ( WatchedItem $watchedItem, string[] $recentChangeInfo ),
 	 *         where $recentChangeInfo contains the following keys:
 	 *         - 'rc_id',
 	 *         - 'rc_namespace',
@@ -256,7 +264,7 @@ class WatchedItemQueryService {
 	/**
 	 * For simple listing of user's watchlist items, see WatchedItemStore::getWatchedItemsForUser
 	 *
-	 * @param User $user
+	 * @param UserIdentity $user
 	 * @param array $options Allowed keys:
 	 *        'sort'         => string optional sorting by namespace ID and title
 	 *                          one of the self::SORT_* constants
@@ -272,8 +280,8 @@ class WatchedItemQueryService {
 	 *                          specified using the form option
 	 * @return WatchedItem[]
 	 */
-	public function getWatchedItemsForUser( User $user, array $options = [] ) {
-		if ( $user->isAnon() ) {
+	public function getWatchedItemsForUser( UserIdentity $user, array $options = [] ) {
+		if ( !$user->isRegistered() ) {
 			// TODO: should this just return an empty array or rather complain loud at this point
 			// as e.g. ApiBase::getWatchlistUser does?
 			return [];
@@ -460,11 +468,12 @@ class WatchedItemQueryService {
 		return $conds;
 	}
 
-	private function getWatchlistOwnerId( User $user, array $options ) {
+	private function getWatchlistOwnerId( UserIdentity $user, array $options ) {
 		if ( array_key_exists( 'watchlistOwner', $options ) ) {
 			/** @var User $watchlistOwner */
 			$watchlistOwner = $options['watchlistOwner'];
-			$ownersToken = $watchlistOwner->getOption( 'watchlisttoken' );
+			$ownersToken =
+				$watchlistOwner->getOption( 'watchlisttoken' );
 			$token = $options['watchlistOwnerToken'];
 			if ( $ownersToken == '' || !hash_equals( $ownersToken, $token ) ) {
 				throw ApiUsageException::newWithMessage( null, 'apierror-bad-watchlist-token', 'bad_wltoken' );
@@ -546,7 +555,7 @@ class WatchedItemQueryService {
 		return $conds;
 	}
 
-	private function getUserRelatedConds( IDatabase $db, User $user, array $options ) {
+	private function getUserRelatedConds( IDatabase $db, UserIdentity $user, array $options ) {
 		if ( !array_key_exists( 'onlyByUser', $options ) && !array_key_exists( 'notByUser', $options ) ) {
 			return [];
 		}
@@ -563,10 +572,12 @@ class WatchedItemQueryService {
 
 		// Avoid brute force searches (T19342)
 		$bitmask = 0;
-		if ( !$user->isAllowed( 'deletedhistory' ) ) {
-			$bitmask = Revision::DELETED_USER;
-		} elseif ( !$user->isAllowedAny( 'suppressrevision', 'viewsuppressed' ) ) {
-			$bitmask = Revision::DELETED_USER | Revision::DELETED_RESTRICTED;
+		if ( !$this->permissionManager->userHasRight( $user, 'deletedhistory' ) ) {
+			$bitmask = RevisionRecord::DELETED_USER;
+		} elseif ( !$this->permissionManager
+			->userHasAnyRight( $user, 'suppressrevision', 'viewsuppressed' )
+		) {
+			$bitmask = RevisionRecord::DELETED_USER | RevisionRecord::DELETED_RESTRICTED;
 		}
 		if ( $bitmask ) {
 			$conds[] = $db->bitAnd( 'rc_deleted', $bitmask ) . " != $bitmask";
@@ -575,13 +586,15 @@ class WatchedItemQueryService {
 		return $conds;
 	}
 
-	private function getExtraDeletedPageLogEntryRelatedCond( IDatabase $db, User $user ) {
+	private function getExtraDeletedPageLogEntryRelatedCond( IDatabase $db, UserIdentity $user ) {
 		// LogPage::DELETED_ACTION hides the affected page, too. So hide those
 		// entirely from the watchlist, or someone could guess the title.
 		$bitmask = 0;
-		if ( !$user->isAllowed( 'deletedhistory' ) ) {
+		if ( !$this->permissionManager->userHasRight( $user, 'deletedhistory' ) ) {
 			$bitmask = LogPage::DELETED_ACTION;
-		} elseif ( !$user->isAllowedAny( 'suppressrevision', 'viewsuppressed' ) ) {
+		} elseif ( !$this->permissionManager
+			->userHasAnyRight( $user, 'suppressrevision', 'viewsuppressed' )
+		) {
 			$bitmask = LogPage::DELETED_ACTION | LogPage::DELETED_RESTRICTED;
 		}
 		if ( $bitmask ) {
@@ -613,7 +626,9 @@ class WatchedItemQueryService {
 		);
 	}
 
-	private function getWatchedItemsForUserQueryConds( IDatabase $db, User $user, array $options ) {
+	private function getWatchedItemsForUserQueryConds(
+		IDatabase $db, UserIdentity $user, array $options
+	) {
 		$conds = [ 'wl_user' => $user->getId() ];
 		if ( $options['namespaceIds'] ) {
 			$conds['wl_namespace'] = array_map( 'intval', $options['namespaceIds'] );
