@@ -294,7 +294,7 @@ $magicWords = [
 ];
 
 $bookstoreList = [
-	'Amazon.fr'    => 'http://www.amazon.fr/exec/obidos/ISBN=$1',
+	'Amazon.fr'    => 'https://www.amazon.fr/exec/obidos/ISBN=$1',
 	'alapage.fr'   => 'http://www.alapage.com/mx/?tp=F&type=101&l_isbn=$1&donnee_appel=ALASQ&devise=&',
 	'fnac.com'     => 'http://www3.fnac.com/advanced/book.do?isbn=$1',
 	'chapitre.com' => 'http://www.chapitre.com/frame_rec.asp?isbn=$1',
@@ -316,5 +316,4 @@ $dateFormats = [
 	'ymd both' => 'Y F j à H:i',
 ];
 
-$separatorTransformTable = [ ',' => "\xc2\xa0", '.' => ',' ];
-
+$separatorTransformTable = [ ',' => "\u{00A0}", '.' => ',' ];
