@@ -48,22 +48,27 @@ $specialPageAliases = [
 	'ApiHelp'                   => [ 'עזרת_API' ],
 	'ApiSandbox'                => [ 'ארגז_חול_של_API' ],
 	'Ancientpages'              => [ 'דפים_מוזנחים' ],
+	'AutoblockList'             => [ 'חסימות_אוטומטיות', 'רשימת_חסימות_אוטומטיות' ],
 	'Badtitle'                  => [ 'כותרת_שגויה' ],
 	'Blankpage'                 => [ 'דף_ריק' ],
 	'Block'                     => [ 'חסימה', 'חסימת_כתובת', 'חסימת_משתמש' ],
 	'Booksources'               => [ 'משאבי_ספרות', 'משאבי_ספרות_חיצוניים' ],
+	'BotPasswords'              => [ 'סיסמאות_בוט' ],
 	'BrokenRedirects'           => [ 'הפניות_לא_תקינות', 'הפניות_שבורות' ],
 	'Categories'                => [ 'קטגוריות', 'רשימת_קטגוריות' ],
+	'ChangeContentModel'        => [ 'שינוי_מודל_התוכן' ],
+	'ChangeCredentials'         => [ 'שינוי_נתוני_ההזדהות' ],
 	'ChangeEmail'               => [ 'שינוי_דואר_אלקטרוני', 'שינוי_דוא"ל' ],
 	'ChangePassword'            => [ 'שינוי_סיסמה' ],
 	'ComparePages'              => [ 'השוואת_דפים' ],
 	'Confirmemail'              => [ 'אימות_כתובת_דואר' ],
 	'Contributions'             => [ 'תרומות', 'תרומות_המשתמש' ],
-	'CreateAccount'             => [ 'הרשמה_לחשבון' ],
+	'CreateAccount'             => [ 'הרשמה_לחשבון', 'יצירת_חשבון' ],
 	'Deadendpages'              => [ 'דפים_ללא_קישורים' ],
 	'DeletedContributions'      => [ 'תרומות_מחוקות' ],
 	'Diff'                      => [ 'הבדלים', 'הבדל' ],
 	'DoubleRedirects'           => [ 'הפניות_כפולות' ],
+	'EditTags'                  => [ 'עריכת_תגיות' ],
 	'EditWatchlist'             => [ 'עריכת_רשימת_המעקב' ],
 	'Emailuser'                 => [ 'שליחת_דואר_למשתמש' ],
 	'ExpandTemplates'           => [ 'פריסת_תבניות' ],
@@ -71,15 +76,18 @@ $specialPageAliases = [
 	'Fewestrevisions'           => [ 'הגרסאות_המעטות_ביותר', 'הדפים_בעלי_מספר_העריכות_הנמוך_ביותר' ],
 	'FileDuplicateSearch'       => [ 'חיפוש_קבצים_כפולים' ],
 	'Filepath'                  => [ 'נתיב_לקובץ' ],
+	'GoToInterwiki'             => [ 'מעבר_לאתר_אחר' ],
 	'Import'                    => [ 'ייבוא', 'ייבוא_דפים' ],
 	'Invalidateemail'           => [ 'ביטול_דואר' ],
 	'JavaScriptTest'            => [ 'בדיקת_JavaScript' ],
 	'BlockList'                 => [ 'רשימת_חסומים', 'רשימת_משתמשים_חסומים', 'משתמשים_חסומים' ],
 	'LinkSearch'                => [ 'חיפוש_קישורים_חיצוניים' ],
+	'LinkAccounts'              => [ 'קישור_חשבונות' ],
 	'Listadmins'                => [ 'רשימת_מפעילים' ],
 	'Listbots'                  => [ 'רשימת_בוטים' ],
 	'Listfiles'                 => [ 'רשימת_קבצים', 'רשימת_תמונות', 'קבצים', 'תמונות' ],
 	'Listgrouprights'           => [ 'רשימת_הרשאות_לקבוצה' ],
+	'Listgrants'                => [ 'רשימת_זיכיונות', 'זיכיונות' ],
 	'Listredirects'             => [ 'רשימת_הפניות', 'הפניות' ],
 	'ListDuplicatedFiles'       => [ 'רשימת_קבצים_כפולים' ],
 	'Listusers'                 => [ 'רשימת_משתמשים', 'משתמשים' ],
@@ -106,7 +114,9 @@ $specialPageAliases = [
 	'Newimages'                 => [ 'קבצים_חדשים', 'תמונות_חדשות', 'גלריית_קבצים_חדשים', 'גלריית_תמונות_חדשות' ],
 	'Newpages'                  => [ 'דפים_חדשים' ],
 	'PagesWithProp'             => [ 'דפים_עם_מאפיינים', 'דפים_לפי_מאפיינים' ],
+	'PageData'                  => [ 'מידע_על_הדף' ],
 	'PageLanguage'              => [ 'שפת_הדף' ],
+	'PasswordPolicies'          => [ 'מדיניות_הסיסמאות' ],
 	'PasswordReset'             => [ 'איפוס_סיסמה' ],
 	'PermanentLink'             => [ 'קישור_קבוע' ],
 	'Preferences'               => [ 'העדפות', 'ההעדפות_שלי' ],
@@ -116,9 +126,11 @@ $specialPageAliases = [
 	'Randompage'                => [ 'אקראי', 'דף_אקראי' ],
 	'RandomInCategory'          => [ 'דף_אקראי_בקטגוריה' ],
 	'Randomredirect'            => [ 'הפניה_אקראית' ],
+	'Randomrootpage'            => [ 'דף_בסיס_אקראי' ],
 	'Recentchanges'             => [ 'שינויים_אחרונים' ],
 	'Recentchangeslinked'       => [ 'שינויים_בדפים_המקושרים' ],
 	'Redirect'                  => [ 'הפניה' ],
+	'RemoveCredentials'         => [ 'הסרת_נתוני_ההזדהות' ],
 	'ResetTokens'               => [ 'איפוס_אסימונים' ],
 	'Revisiondelete'            => [ 'מחיקת_ושחזור_גרסאות' ],
 	'RunJobs'                   => [ 'הרצת_משימות' ],
@@ -134,6 +146,7 @@ $specialPageAliases = [
 	'Uncategorizedpages'        => [ 'דפים_חסרי_קטגוריה' ],
 	'Uncategorizedtemplates'    => [ 'תבניות_חסרות_קטגוריות' ],
 	'Undelete'                  => [ 'צפייה_בדפים_מחוקים' ],
+	'UnlinkAccounts'            => [ 'ביטול_הקישור_בין_חשבונות' ],
 	'Unlockdb'                  => [ 'שחרור_בסיס_הנתונים' ],
 	'Unusedcategories'          => [ 'קטגוריות_שאינן_בשימוש' ],
 	'Unusedimages'              => [ 'קבצים_שאינם_בשימוש', 'תמונות_שאינן_בשימוש' ],
