@@ -4,7 +4,7 @@
  * @copyright 2011-2015 MediaWiki Widgets Team and others; see AUTHORS.txt
  * @license The MIT License (MIT); see LICENSE.txt
  */
-( function ( $, mw ) {
+( function () {
 
 	/**
 	 * Like TitleInputWidget, but the namespace has to be input through a separate dropdown field.
@@ -118,4 +118,4 @@
 		return this;
 	};
 
-}( jQuery, mediaWiki ) );
+}() );
