@@ -4,7 +4,9 @@
  * @copyright 2011-2015 MediaWiki Widgets Team and others; see AUTHORS.txt
  * @license The MIT License (MIT); see LICENSE.txt
  */
-( function ( $, mw ) {
+( function () {
+
+	var trimByteLength = require( 'mediawiki.String' ).trimByteLength;
 
 	/**
 	 * Creates an mw.widgets.TitleInputWidget object.
@@ -130,10 +132,10 @@
 		// Parent method
 		value = mw.widgets.TitleInputWidget.parent.prototype.cleanUpValue.call( this, value );
 
-		return $.trimByteLength( this.value, value, this.maxLength, function ( value ) {
-			var title = widget.getTitle( value );
+		return trimByteLength( this.value, value, this.maxLength, function ( value ) {
+			var title = widget.getMWTitle( value );
 			return title ? title.getMain() : value;
 		} ).newVal;
 	};
 
-}( jQuery, mediaWiki ) );
+}() );
