@@ -74,12 +74,8 @@ class GlobalTest extends MediaWikiTestCase {
 		$this->assertFalse(
 			wfRandomString() == wfRandomString()
 		);
-		$this->assertEquals(
-			strlen( wfRandomString( 10 ) ), 10
-		);
-		$this->assertTrue(
-			preg_match( '/^[0-9a-f]+$/i', wfRandomString() ) === 1
-		);
+		$this->assertSame( 10, strlen( wfRandomString( 10 ) ), 'length' );
+		$this->assertSame( 1, preg_match( '/^[0-9a-f]+$/i', wfRandomString() ), 'pattern' );
 	}
 
 	/**
@@ -124,9 +120,6 @@ class GlobalTest extends MediaWikiTestCase {
 		fwrite( $f, 'Message' );
 		fclose( $f );
 
-		// Reset the service to avoid cached results
-		$this->overrideMwServices();
-
 		$this->assertTrue( wfReadOnly() );
 		$this->assertTrue( wfReadOnly() ); # Check cached
 	}
@@ -141,7 +134,6 @@ class GlobalTest extends MediaWikiTestCase {
 		$this->setMwGlobals( [
 			'wgReadOnly' => 'reason'
 		] );
-		$this->overrideMwServices();
 
 		$this->assertSame( 'reason', wfReadOnlyReason() );
 	}
@@ -325,33 +317,35 @@ class GlobalTest extends MediaWikiTestCase {
 		] );
 		$this->setLogger( 'wfDebug', new LegacyLogger( 'wfDebug' ) );
 
+		unlink( $debugLogFile );
 		wfDebug( "This is a normal string" );
 		$this->assertEquals( "This is a normal string\n", file_get_contents( $debugLogFile ) );
-		unlink( $debugLogFile );
 
+		unlink( $debugLogFile );
 		wfDebug( "This is nöt an ASCII string" );
 		$this->assertEquals( "This is nöt an ASCII string\n", file_get_contents( $debugLogFile ) );
-		unlink( $debugLogFile );
 
+		unlink( $debugLogFile );
 		wfDebug( "\00305This has böth UTF and control chars\003" );
 		$this->assertEquals(
 			" 05This has böth UTF and control chars \n",
 			file_get_contents( $debugLogFile )
 		);
-		unlink( $debugLogFile );
 
+		unlink( $debugLogFile );
 		wfDebugMem();
 		$this->assertGreaterThan(
 			1000,
 			preg_replace( '/\D/', '', file_get_contents( $debugLogFile ) )
 		);
-		unlink( $debugLogFile );
 
+		unlink( $debugLogFile );
 		wfDebugMem( true );
 		$this->assertGreaterThan(
 			1000000,
 			preg_replace( '/\D/', '', file_get_contents( $debugLogFile ) )
 		);
+
 		unlink( $debugLogFile );
 	}
 
@@ -720,6 +714,7 @@ class GlobalTest extends MediaWikiTestCase {
 	 */
 	public function testWfGlobalCacheKey() {
 		$cache = ObjectCache::getLocalClusterInstance();
+		$this->hideDeprecated( 'wfGlobalCacheKey' );
 		$this->assertEquals(
 			$cache->makeGlobalKey( 'foo', 123, 'bar' ),
 			wfGlobalCacheKey( 'foo', 123, 'bar' )
