@@ -4,7 +4,6 @@ namespace MediaWiki\Tests\Revision;
 
 use ActorMigration;
 use CommentStore;
-use MediaWiki\Logger\Spi as LoggerSpi;
 use MediaWiki\Revision\RevisionStore;
 use MediaWiki\Revision\RevisionStoreFactory;
 use MediaWiki\Revision\SlotRoleRegistry;
@@ -13,7 +12,6 @@ use MediaWiki\Storage\BlobStoreFactory;
 use MediaWiki\Storage\NameTableStore;
 use MediaWiki\Storage\NameTableStoreFactory;
 use MediaWiki\Storage\SqlBlobStore;
-use MediaWikiTestCase;
 use Psr\Log\LoggerInterface;
 use Psr\Log\NullLogger;
 use WANObjectCache;
@@ -21,7 +19,7 @@ use Wikimedia\Rdbms\ILBFactory;
 use Wikimedia\Rdbms\ILoadBalancer;
 use Wikimedia\TestingAccessWrapper;
 
-class RevisionStoreFactoryTest extends MediaWikiTestCase {
+class RevisionStoreFactoryTest extends \MediaWikiIntegrationTestCase {
 
 	/**
 	 * @covers \MediaWiki\Revision\RevisionStoreFactory::__construct
@@ -35,8 +33,8 @@ class RevisionStoreFactoryTest extends MediaWikiTestCase {
 			$this->getHashWANObjectCache(),
 			$this->getMockCommentStore(),
 			ActorMigration::newMigration(),
-			MIGRATION_OLD,
-			$this->getMockLoggerSpi(),
+			MIGRATION_NEW,
+			new NullLogger(),
 			true
 		);
 		$this->assertTrue( true );
@@ -46,8 +44,6 @@ class RevisionStoreFactoryTest extends MediaWikiTestCase {
 		yield [ true ];
 		yield [ false ];
 		yield [ 'somewiki' ];
-		yield [ 'somewiki', MIGRATION_OLD , false ];
-		yield [ 'somewiki', MIGRATION_NEW , true ];
 	}
 
 	/**
@@ -55,8 +51,8 @@ class RevisionStoreFactoryTest extends MediaWikiTestCase {
 	 * @covers \MediaWiki\Revision\RevisionStoreFactory::getRevisionStore
 	 */
 	public function testGetRevisionStore(
-		$wikiId,
-		$mcrMigrationStage = MIGRATION_OLD,
+		$dbDomain,
+		$mcrMigrationStage = MIGRATION_NEW,
 		$contentHandlerUseDb = true
 	) {
 		$lbFactory = $this->getMockLoadBalancerFactory();
@@ -66,7 +62,7 @@ class RevisionStoreFactoryTest extends MediaWikiTestCase {
 		$cache = $this->getHashWANObjectCache();
 		$commentStore = $this->getMockCommentStore();
 		$actorMigration = ActorMigration::newMigration();
-		$loggerProvider = $this->getMockLoggerSpi();
+		$logger = new NullLogger();
 
 		$factory = new RevisionStoreFactory(
 			$lbFactory,
@@ -77,18 +73,18 @@ class RevisionStoreFactoryTest extends MediaWikiTestCase {
 			$commentStore,
 			$actorMigration,
 			$mcrMigrationStage,
-			$loggerProvider,
+			$logger,
 			$contentHandlerUseDb
 		);
 
-		$store = $factory->getRevisionStore( $wikiId );
+		$store = $factory->getRevisionStore( $dbDomain );
 		$wrapper = TestingAccessWrapper::newFromObject( $store );
 
 		// ensure the correct object type is returned
 		$this->assertInstanceOf( RevisionStore::class, $store );
 
 		// ensure the RevisionStore is for the given wikiId
-		$this->assertSame( $wikiId, $wrapper->wikiId );
+		$this->assertSame( $dbDomain, $wrapper->dbDomain );
 
 		// ensure all other required services are correctly set
 		$this->assertSame( $cache, $wrapper->cache );
@@ -151,13 +147,10 @@ class RevisionStoreFactoryTest extends MediaWikiTestCase {
 	}
 
 	/**
-	 * @return \PHPUnit_Framework_MockObject_MockObject|SlotRoleRegistry
+	 * @return SlotRoleRegistry
 	 */
 	private function getMockSlotRoleRegistry() {
-		$mock = $this->getMockBuilder( SlotRoleRegistry::class )
-			->disableOriginalConstructor()->getMock();
-
-		return $mock;
+		return $this->createMock( SlotRoleRegistry::class );
 	}
 
 	/**
@@ -182,16 +175,4 @@ class RevisionStoreFactoryTest extends MediaWikiTestCase {
 		return new WANObjectCache( [ 'cache' => new \HashBagOStuff() ] );
 	}
 
-	/**
-	 * @return \PHPUnit_Framework_MockObject_MockObject|LoggerSpi
-	 */
-	private function getMockLoggerSpi() {
-		$mock = $this->getMock( LoggerSpi::class );
-
-		$mock->method( 'getLogger' )
-			->willReturn( new NullLogger() );
-
-		return $mock;
-	}
-
 }
