@@ -1,4 +1,5 @@
 <?php
+
 use MediaWiki\MediaWikiServices;
 use MediaWiki\Revision\MutableRevisionRecord;
 use MediaWiki\Revision\RevisionStore;
@@ -91,11 +92,8 @@ abstract class RevisionDbTestBase extends MediaWikiTestCase {
 		$this->setMwGlobals( [
 			'wgMultiContentRevisionSchemaMigrationStage' => $this->getMcrMigrationStage(),
 			'wgContentHandlerUseDB' => $this->getContentHandlerUseDB(),
-			'wgActorTableSchemaMigrationStage' => SCHEMA_COMPAT_NEW,
 		] );
 
-		$this->overrideMwServices();
-
 		if ( !$this->testPage ) {
 			/**
 			 * We have to create a new page for each subclass as the page creation may result
@@ -182,7 +180,8 @@ abstract class RevisionDbTestBase extends MediaWikiTestCase {
 			( $model === null || $model === CONTENT_MODEL_WIKITEXT )
 		) {
 			$ns = $this->getDefaultWikitextNS();
-			$titleString = MWNamespace::getCanonicalName( $ns ) . ':' . $titleString;
+			$titleString = MediaWikiServices::getInstance()->getNamespaceInfo()->
+				getCanonicalName( $ns ) . ':' . $titleString;
 		}
 
 		$title = Title::newFromText( $titleString );
@@ -625,6 +624,34 @@ abstract class RevisionDbTestBase extends MediaWikiTestCase {
 		$this->assertEquals( $latestRevision, $newRevision->getPrevious()->getId() );
 	}
 
+	/**
+	 * @covers Title::getPreviousRevisionID
+	 * @covers Title::getRelativeRevisionID
+	 * @covers MediaWiki\Revision\RevisionStore::getPreviousRevision
+	 * @covers MediaWiki\Revision\RevisionStore::getRelativeRevision
+	 */
+	public function testTitleGetPreviousRevisionID() {
+		$oldestId = $this->testPage->getOldestRevision()->getId();
+		$latestId = $this->testPage->getLatest();
+
+		$title = $this->testPage->getTitle();
+
+		$this->assertFalse( $title->getPreviousRevisionID( $oldestId ) );
+
+		$this->testPage->doEditContent( new WikitextContent( __METHOD__ ), __METHOD__ );
+		$newId = $this->testPage->getRevision()->getId();
+
+		$this->assertEquals( $latestId, $title->getPreviousRevisionID( $newId ) );
+	}
+
+	/**
+	 * @covers Title::getPreviousRevisionID
+	 * @covers Title::getRelativeRevisionID
+	 */
+	public function testTitleGetPreviousRevisionID_invalid() {
+		$this->assertFalse( $this->testPage->getTitle()->getPreviousRevisionID( 123456789 ) );
+	}
+
 	/**
 	 * @covers Revision::getNext
 	 */
@@ -640,6 +667,33 @@ abstract class RevisionDbTestBase extends MediaWikiTestCase {
 		$this->assertEquals( $rev2->getId(), $rev1->getNext()->getId() );
 	}
 
+	/**
+	 * @covers Title::getNextRevisionID
+	 * @covers Title::getRelativeRevisionID
+	 * @covers MediaWiki\Revision\RevisionStore::getNextRevision
+	 * @covers MediaWiki\Revision\RevisionStore::getRelativeRevision
+	 */
+	public function testTitleGetNextRevisionID() {
+		$title = $this->testPage->getTitle();
+
+		$origId = $this->testPage->getLatest();
+
+		$this->assertFalse( $title->getNextRevisionID( $origId ) );
+
+		$this->testPage->doEditContent( new WikitextContent( __METHOD__ ), __METHOD__ );
+		$newId = $this->testPage->getLatest();
+
+		$this->assertSame( $this->testPage->getLatest(), $title->getNextRevisionID( $origId ) );
+	}
+
+	/**
+	 * @covers Title::getNextRevisionID
+	 * @covers Title::getRelativeRevisionID
+	 */
+	public function testTitleGetNextRevisionID_invalid() {
+		$this->assertFalse( $this->testPage->getTitle()->getNextRevisionID( 123456789 ) );
+	}
+
 	/**
 	 * @covers Revision::newNullRevision
 	 */
@@ -1411,7 +1465,7 @@ abstract class RevisionDbTestBase extends MediaWikiTestCase {
 			$rev->getPage(),
 			$rev->getId()
 		);
-		$cache->delete( $key, WANObjectCache::HOLDOFF_NONE );
+		$cache->delete( $key, WANObjectCache::HOLDOFF_TTL_NONE );
 		$this->assertFalse( $cache->get( $key ) );
 
 		++$now;
@@ -1482,8 +1536,7 @@ abstract class RevisionDbTestBase extends MediaWikiTestCase {
 	public function testUserCanBitfield( $bitField, $field, $userGroups, $title, $expected ) {
 		$title = Title::newFromText( $title );
 
-		$this->setMwGlobals(
-			'wgGroupPermissions',
+		$this->setGroupPermissions(
 			[
 				'sysop' => [
 					'deletedtext' => true,
@@ -1535,8 +1588,7 @@ abstract class RevisionDbTestBase extends MediaWikiTestCase {
 	 * @covers Revision::userCan
 	 */
 	public function testUserCan( $bitField, $field, $userGroups, $expected ) {
-		$this->setMwGlobals(
-			'wgGroupPermissions',
+		$this->setGroupPermissions(
 			[
 				'sysop' => [
 					'deletedtext' => true,
@@ -1585,11 +1637,7 @@ abstract class RevisionDbTestBase extends MediaWikiTestCase {
 		$this->assertSame( $expected, $rev->getTextId() );
 	}
 
-	public function provideGetRevisionText() {
-		yield [
-			[ 'text' ]
-		];
-	}
+	abstract public function provideGetRevisionText();
 
 	/**
 	 * @dataProvider provideGetRevisionText
