@@ -1,4 +1,5 @@
 <?php
+
 use MediaWiki\Tests\Revision\McrWriteBothSchemaOverride;
 
 /**
@@ -43,4 +44,9 @@ class RevisionMcrWriteBothDbTest extends RevisionDbTestBase {
 		yield [ $row, 789 ];
 	}
 
+	public function provideGetRevisionText() {
+		yield [
+			[ 'text' ]
+		];
+	}
 }
