@@ -13,34 +13,34 @@ use Wikimedia\TestingAccessWrapper;
  */
 class BlobStoreFactoryTest extends MediaWikiTestCase {
 
-	public function provideWikiIds() {
+	public function provideDbDomains() {
 		yield [ false ];
 		yield [ 'someWiki' ];
 	}
 
 	/**
-	 * @dataProvider provideWikiIds
+	 * @dataProvider provideDbDomains
 	 */
-	public function testNewBlobStore( $wikiId ) {
+	public function testNewBlobStore( $dbDomain ) {
 		$factory = MediaWikiServices::getInstance()->getBlobStoreFactory();
-		$store = $factory->newBlobStore( $wikiId );
+		$store = $factory->newBlobStore( $dbDomain );
 		$this->assertInstanceOf( BlobStore::class, $store );
 
 		// This only works as we currently know this is a SqlBlobStore object
 		$wrapper = TestingAccessWrapper::newFromObject( $store );
-		$this->assertEquals( $wikiId, $wrapper->wikiId );
+		$this->assertEquals( $dbDomain, $wrapper->dbDomain );
 	}
 
 	/**
-	 * @dataProvider provideWikiIds
+	 * @dataProvider provideDbDomains
 	 */
-	public function testNewSqlBlobStore( $wikiId ) {
+	public function testNewSqlBlobStore( $dbDomain ) {
 		$factory = MediaWikiServices::getInstance()->getBlobStoreFactory();
-		$store = $factory->newSqlBlobStore( $wikiId );
+		$store = $factory->newSqlBlobStore( $dbDomain );
 		$this->assertInstanceOf( SqlBlobStore::class, $store );
 
 		$wrapper = TestingAccessWrapper::newFromObject( $store );
-		$this->assertEquals( $wikiId, $wrapper->wikiId );
+		$this->assertEquals( $dbDomain, $wrapper->dbDomain );
 	}
 
 }
