@@ -1,4 +1,5 @@
 <?php
+
 use Wikimedia\ScopedCallback;
 
 /**
@@ -27,10 +28,16 @@ class RecentChangeTest extends MediaWikiTestCase {
 	 * @covers RecentChange::loadFromRow
 	 */
 	public function testNewFromRow() {
+		$user = $this->getTestUser()->getUser();
+		$actorId = $user->getActorId();
+
 		$row = new stdClass();
 		$row->rc_foo = 'AAA';
 		$row->rc_timestamp = '20150921134808';
 		$row->rc_deleted = 'bar';
+		$row->rc_comment_text = 'comment';
+		$row->rc_comment_data = null;
+		$row->rc_user = $user->getId();
 
 		$rc = RecentChange::newFromRow( $row );
 
@@ -38,6 +45,36 @@ class RecentChangeTest extends MediaWikiTestCase {
 			'rc_foo' => 'AAA',
 			'rc_timestamp' => '20150921134808',
 			'rc_deleted' => 'bar',
+			'rc_comment' => 'comment',
+			'rc_comment_text' => 'comment',
+			'rc_comment_data' => null,
+			'rc_user' => $user->getId(),
+			'rc_user_text' => $user->getName(),
+			'rc_actor' => $actorId,
+		];
+		$this->assertEquals( $expected, $rc->getAttributes() );
+
+		$row = new stdClass();
+		$row->rc_foo = 'AAA';
+		$row->rc_timestamp = '20150921134808';
+		$row->rc_deleted = 'bar';
+		$row->rc_comment = 'comment';
+		$row->rc_user = $user->getId();
+
+		Wikimedia\suppressWarnings();
+		$rc = RecentChange::newFromRow( $row );
+		Wikimedia\restoreWarnings();
+
+		$expected = [
+			'rc_foo' => 'AAA',
+			'rc_timestamp' => '20150921134808',
+			'rc_deleted' => 'bar',
+			'rc_comment' => 'comment',
+			'rc_comment_text' => 'comment',
+			'rc_comment_data' => null,
+			'rc_user' => $user->getId(),
+			'rc_user_text' => $user->getName(),
+			'rc_actor' => $actorId,
 		];
 		$this->assertEquals( $expected, $rc->getAttributes() );
 	}
