@@ -1,5 +1,6 @@
 <?php
 
+use MediaWiki\MediaWikiServices;
 use Wikimedia\TestingAccessWrapper;
 
 /**
@@ -169,6 +170,19 @@ class ChangesListSpecialPageTest extends AbstractChangesListSpecialPageTestCase
 		);
 	}
 
+	public function testRcNsFilterAssociatedSpecial() {
+		$this->assertConditions(
+			[ # expected
+			  "rc_namespace IN ('-1','0','1')",
+			],
+			[
+				'namespace' => '1;-1',
+				'associated' => 1,
+			],
+			"rc conditions with associated and special namespace"
+		);
+	}
+
 	public function testRcNsFilterMultipleAssociatedInvert() {
 		$this->assertConditions(
 			[ # expected
@@ -196,48 +210,25 @@ class ChangesListSpecialPageTest extends AbstractChangesListSpecialPageTestCase
 		);
 	}
 
-	public function testRcHidemyselfFilter() {
-		$this->setMwGlobals( 'wgActorTableSchemaMigrationStage', SCHEMA_COMPAT_NEW );
-		$this->overrideMwServices();
-
-		$user = $this->getTestUser()->getUser();
-		$user->getActorId( wfGetDB( DB_MASTER ) );
+	public function testRcNsFilterAllContents() {
+		$namespaces = MediaWikiServices::getInstance()->getNamespaceInfo()->getSubjectNamespaces();
 		$this->assertConditions(
 			[ # expected
-				"NOT((rc_actor = '{$user->getActorId()}'))",
+				'rc_namespace IN (' . $this->db->makeList( $namespaces ) . ')',
 			],
 			[
-				'hidemyself' => 1,
+				'namespace' => 'all-contents',
 			],
-			"rc conditions: hidemyself=1 (logged in)",
-			$user
-		);
-
-		$user = User::newFromName( '10.11.12.13', false );
-		$id = $user->getActorId( wfGetDB( DB_MASTER ) );
-		$this->assertConditions(
-			[ # expected
-				"NOT((rc_actor = '{$user->getActorId()}'))",
-			],
-			[
-				'hidemyself' => 1,
-			],
-			"rc conditions: hidemyself=1 (anon)",
-			$user
+			"rc conditions with all-contents"
 		);
 	}
 
-	public function testRcHidemyselfFilter_old() {
-		$this->setMwGlobals(
-			'wgActorTableSchemaMigrationStage', SCHEMA_COMPAT_WRITE_BOTH | SCHEMA_COMPAT_READ_OLD
-		);
-		$this->overrideMwServices();
-
+	public function testRcHidemyselfFilter() {
 		$user = $this->getTestUser()->getUser();
 		$user->getActorId( wfGetDB( DB_MASTER ) );
 		$this->assertConditions(
 			[ # expected
-				"NOT((rc_user = '{$user->getId()}'))",
+				"NOT((rc_actor = '{$user->getActorId()}'))",
 			],
 			[
 				'hidemyself' => 1,
@@ -250,7 +241,7 @@ class ChangesListSpecialPageTest extends AbstractChangesListSpecialPageTestCase
 		$id = $user->getActorId( wfGetDB( DB_MASTER ) );
 		$this->assertConditions(
 			[ # expected
-				"NOT((rc_user_text = '10.11.12.13'))",
+				"NOT((rc_actor = '{$user->getActorId()}'))",
 			],
 			[
 				'hidemyself' => 1,
@@ -261,9 +252,6 @@ class ChangesListSpecialPageTest extends AbstractChangesListSpecialPageTestCase
 	}
 
 	public function testRcHidebyothersFilter() {
-		$this->setMwGlobals( 'wgActorTableSchemaMigrationStage', SCHEMA_COMPAT_NEW );
-		$this->overrideMwServices();
-
 		$user = $this->getTestUser()->getUser();
 		$user->getActorId( wfGetDB( DB_MASTER ) );
 		$this->assertConditions(
@@ -291,39 +279,6 @@ class ChangesListSpecialPageTest extends AbstractChangesListSpecialPageTestCase
 		);
 	}
 
-	public function testRcHidebyothersFilter_old() {
-		$this->setMwGlobals(
-			'wgActorTableSchemaMigrationStage', SCHEMA_COMPAT_WRITE_BOTH | SCHEMA_COMPAT_READ_OLD
-		);
-		$this->overrideMwServices();
-
-		$user = $this->getTestUser()->getUser();
-		$user->getActorId( wfGetDB( DB_MASTER ) );
-		$this->assertConditions(
-			[ # expected
-				"(rc_user_text = '{$user->getName()}')",
-			],
-			[
-				'hidebyothers' => 1,
-			],
-			"rc conditions: hidebyothers=1 (logged in)",
-			$user
-		);
-
-		$user = User::newFromName( '10.11.12.13', false );
-		$id = $user->getActorId( wfGetDB( DB_MASTER ) );
-		$this->assertConditions(
-			[ # expected
-				"(rc_user_text = '10.11.12.13')",
-			],
-			[
-				'hidebyothers' => 1,
-			],
-			"rc conditions: hidebyothers=1 (anon)",
-			$user
-		);
-	}
-
 	public function testRcHidepageedits() {
 		$this->assertConditions(
 			[ # expected
@@ -400,6 +355,7 @@ class ChangesListSpecialPageTest extends AbstractChangesListSpecialPageTestCase
 			$user
 		);
 	}
+
 	public function testRcHidepatrolledFilter() {
 		$user = $this->getTestSysop()->getUser();
 		$this->assertConditions(
@@ -526,9 +482,6 @@ class ChangesListSpecialPageTest extends AbstractChangesListSpecialPageTestCase
 	}
 
 	public function testFilterUserExpLevelAllExperienceLevels() {
-		$this->setMwGlobals( 'wgActorTableSchemaMigrationStage', SCHEMA_COMPAT_NEW );
-		$this->overrideMwServices();
-
 		$this->assertConditions(
 			[
 				# expected
@@ -541,28 +494,7 @@ class ChangesListSpecialPageTest extends AbstractChangesListSpecialPageTestCase
 		);
 	}
 
-	public function testFilterUserExpLevelAllExperienceLevels_old() {
-		$this->setMwGlobals(
-			'wgActorTableSchemaMigrationStage', SCHEMA_COMPAT_WRITE_BOTH | SCHEMA_COMPAT_READ_OLD
-		);
-		$this->overrideMwServices();
-
-		$this->assertConditions(
-			[
-				# expected
-				'rc_user != 0',
-			],
-			[
-				'userExpLevel' => 'newcomer;learner;experienced',
-			],
-			"rc conditions: userExpLevel=newcomer;learner;experienced"
-		);
-	}
-
-	public function testFilterUserExpLevelRegistrered() {
-		$this->setMwGlobals( 'wgActorTableSchemaMigrationStage', SCHEMA_COMPAT_NEW );
-		$this->overrideMwServices();
-
+	public function testFilterUserExpLevelRegistered() {
 		$this->assertConditions(
 			[
 				# expected
@@ -575,28 +507,7 @@ class ChangesListSpecialPageTest extends AbstractChangesListSpecialPageTestCase
 		);
 	}
 
-	public function testFilterUserExpLevelRegistrered_old() {
-		$this->setMwGlobals(
-			'wgActorTableSchemaMigrationStage', SCHEMA_COMPAT_WRITE_BOTH | SCHEMA_COMPAT_READ_OLD
-		);
-		$this->overrideMwServices();
-
-		$this->assertConditions(
-			[
-				# expected
-				'rc_user != 0',
-			],
-			[
-				'userExpLevel' => 'registered',
-			],
-			"rc conditions: userExpLevel=registered"
-		);
-	}
-
-	public function testFilterUserExpLevelUnregistrered() {
-		$this->setMwGlobals( 'wgActorTableSchemaMigrationStage', SCHEMA_COMPAT_NEW );
-		$this->overrideMwServices();
-
+	public function testFilterUserExpLevelUnregistered() {
 		$this->assertConditions(
 			[
 				# expected
@@ -609,28 +520,7 @@ class ChangesListSpecialPageTest extends AbstractChangesListSpecialPageTestCase
 		);
 	}
 
-	public function testFilterUserExpLevelUnregistrered_old() {
-		$this->setMwGlobals(
-			'wgActorTableSchemaMigrationStage', SCHEMA_COMPAT_WRITE_BOTH | SCHEMA_COMPAT_READ_OLD
-		);
-		$this->overrideMwServices();
-
-		$this->assertConditions(
-			[
-				# expected
-				'rc_user = 0',
-			],
-			[
-				'userExpLevel' => 'unregistered',
-			],
-			"rc conditions: userExpLevel=unregistered"
-		);
-	}
-
-	public function testFilterUserExpLevelRegistreredOrLearner() {
-		$this->setMwGlobals( 'wgActorTableSchemaMigrationStage', SCHEMA_COMPAT_NEW );
-		$this->overrideMwServices();
-
+	public function testFilterUserExpLevelRegisteredOrLearner() {
 		$this->assertConditions(
 			[
 				# expected
@@ -643,28 +533,7 @@ class ChangesListSpecialPageTest extends AbstractChangesListSpecialPageTestCase
 		);
 	}
 
-	public function testFilterUserExpLevelRegistreredOrLearner_old() {
-		$this->setMwGlobals(
-			'wgActorTableSchemaMigrationStage', SCHEMA_COMPAT_WRITE_BOTH | SCHEMA_COMPAT_READ_OLD
-		);
-		$this->overrideMwServices();
-
-		$this->assertConditions(
-			[
-				# expected
-				'rc_user != 0',
-			],
-			[
-				'userExpLevel' => 'registered;learner',
-			],
-			"rc conditions: userExpLevel=registered;learner"
-		);
-	}
-
-	public function testFilterUserExpLevelUnregistreredOrExperienced() {
-		$this->setMwGlobals( 'wgActorTableSchemaMigrationStage', SCHEMA_COMPAT_NEW );
-		$this->overrideMwServices();
-
+	public function testFilterUserExpLevelUnregisteredOrExperienced() {
 		$conds = $this->buildQuery( [ 'userExpLevel' => 'unregistered;experienced' ] );
 
 		$this->assertRegExp(
@@ -675,22 +544,6 @@ class ChangesListSpecialPageTest extends AbstractChangesListSpecialPageTestCase
 		);
 	}
 
-	public function testFilterUserExpLevelUnregistreredOrExperienced_old() {
-		$this->setMwGlobals(
-			'wgActorTableSchemaMigrationStage', SCHEMA_COMPAT_WRITE_BOTH | SCHEMA_COMPAT_READ_OLD
-		);
-		$this->overrideMwServices();
-
-		$conds = $this->buildQuery( [ 'userExpLevel' => 'unregistered;experienced' ] );
-
-		$this->assertRegExp(
-			'/\(rc_user = 0\) OR '
-				. '\(\(user_editcount >= 500\) AND \(user_registration <= \'[^\']+\'\)\)/',
-			reset( $conds ),
-			"rc conditions: userExpLevel=unregistered;experienced"
-		);
-	}
-
 	public function testFilterUserExpLevel() {
 		$now = time();
 		$this->setMwGlobals( [
