@@ -1,5 +1,8 @@
 <?php
 
+use MediaWiki\Block\DatabaseBlock;
+use MediaWiki\MediaWikiServices;
+
 /**
  * @covers LocalIdLookup
  * @group Database
@@ -20,7 +23,7 @@ class LocalIdLookupTest extends MediaWikiTestCase {
 
 		$sysop = static::getTestSysop()->getUser();
 
-		$block = new Block( [
+		$block = new DatabaseBlock( [
 			'address' => $this->localUsers[2]->getName(),
 			'by' => $sysop->getId(),
 			'reason' => __METHOD__,
@@ -29,7 +32,7 @@ class LocalIdLookupTest extends MediaWikiTestCase {
 		] );
 		$block->insert();
 
-		$block = new Block( [
+		$block = new DatabaseBlock( [
 			'address' => $this->localUsers[3]->getName(),
 			'by' => $sysop->getId(),
 			'reason' => __METHOD__,
@@ -45,12 +48,12 @@ class LocalIdLookupTest extends MediaWikiTestCase {
 
 	public function testLookupCentralIds() {
 		$lookup = new LocalIdLookup();
-
+		$permissionManager = MediaWikiServices::getInstance()->getPermissionManager();
 		$user1 = $this->getLookupUser();
 		$user2 = User::newFromName( 'UTLocalIdLookup2' );
 
-		$this->assertTrue( $user1->isAllowed( 'hideuser' ), 'sanity check' );
-		$this->assertFalse( $user2->isAllowed( 'hideuser' ), 'sanity check' );
+		$this->assertTrue( $permissionManager->userHasRight( $user1, 'hideuser' ), 'sanity check' );
+		$this->assertFalse( $permissionManager->userHasRight( $user2, 'hideuser' ), 'sanity check' );
 
 		$this->assertSame( [], $lookup->lookupCentralIds( [] ) );
 
@@ -74,11 +77,12 @@ class LocalIdLookupTest extends MediaWikiTestCase {
 
 	public function testLookupUserNames() {
 		$lookup = new LocalIdLookup();
+		$permissionManager = MediaWikiServices::getInstance()->getPermissionManager();
 		$user1 = $this->getLookupUser();
 		$user2 = User::newFromName( 'UTLocalIdLookup2' );
 
-		$this->assertTrue( $user1->isAllowed( 'hideuser' ), 'sanity check' );
-		$this->assertFalse( $user2->isAllowed( 'hideuser' ), 'sanity check' );
+		$this->assertTrue( $permissionManager->userHasRight( $user1, 'hideuser' ), 'sanity check' );
+		$this->assertFalse( $permissionManager->userHasRight( $user2, 'hideuser' ), 'sanity check' );
 
 		$this->assertSame( [], $lookup->lookupUserNames( [] ) );
 
