@@ -5,14 +5,18 @@
  * @file
  */
 
-/** Tests for MediaWiki languages/classes/LanguageGa.php */
+/**
+ * Tests for Irish (Gaeilge)
+ *
+ * @covers LanguageGa
+ */
 class LanguageGaTest extends LanguageClassesTestCase {
 	/**
 	 * @dataProvider providePlural
 	 * @covers Language::convertPlural
 	 */
 	public function testPlural( $result, $value ) {
-		$forms = array( 'one', 'two', 'other' );
+		$forms = [ 'one', 'two', 'other' ];
 		$this->assertEquals( $result, $this->getLang()->convertPlural( $value, $forms ) );
 	}
 
@@ -25,11 +29,11 @@ class LanguageGaTest extends LanguageClassesTestCase {
 	}
 
 	public static function providePlural() {
-		return array(
-			array( 'other', 0 ),
-			array( 'one', 1 ),
-			array( 'two', 2 ),
-			array( 'other', 200 ),
-		);
+		return [
+			[ 'other', 0 ],
+			[ 'one', 1 ],
+			[ 'two', 2 ],
+			[ 'other', 200 ],
+		];
 	}
 }
