@@ -1,7 +1,7 @@
 const Page = require( './Page' );
 
 class BlankPage extends Page {
-	get heading() { return browser.element( '#firstHeading' ); }
+	get heading() { return $( '#firstHeading' ); }
 
 	open() {
 		super.openTitle( 'Special:BlankPage', { uselang: 'en' } );
